@@ -74,7 +74,8 @@
 	#+gcl "gcl"
 	#+allegro "acl"
 	#+openmcl "openmcl"
-	#-(or clisp cmu scl sbcl gcl allegro openmcl) "unknownlisp")
+    #+abcl "abcl"
+	#-(or clisp cmu scl sbcl gcl allegro openmcl abcl) "unknownlisp")
 
 (defun configure (&key (interactive t) (verbose nil)
 		  is-win32 
