@@ -27,7 +27,7 @@
 
 (in-package :maxima)
 
-($put '$graphs 1.0 '$version)
+($put '$graphs 2.0 '$version)
 
 (defstruct (graph
 	     (:print-function
@@ -38,6 +38,7 @@
   (order 0)
   (vertices ())
   (vertex-labels (make-hash-table))
+  (vertex-positions)
   (edges ())
   (edge-weights (make-hash-table :test #'equal))
   (neighbors (make-hash-table)))
@@ -52,21 +53,22 @@
   (vertices ())
   (vertex-labels (make-hash-table))
   (edges ())
+  (vertex-positions)
   (edge-weights (make-hash-table :test #'equal))
   (in-neighbors (make-hash-table))
   (out-neighbors (make-hash-table)))
 
 (defun require-graph (m ar gr)
-  (if (not (graph-p gr))
-      ($error (format nil "Argument ~a to `~a' is not a graph:" ar m) gr)))
+  (unless (graph-p gr)
+    ($error (format nil "Argument ~a to `~a' is not a graph:" ar m) gr)))
 
 (defun require-digraph (m ar gr)
-  (if (not (digraph-p gr))
-      ($error (format nil "Argument ~a to `~a' is not a graph:" ar m) gr)))
+  (unless (digraph-p gr)
+    ($error (format nil "Argument ~a to `~a' is not a graph:" ar m) gr)))
 
 (defun require-graph-or-digraph (m ar gr)
-  (if (not (or (graph-p gr) (digraph-p gr)))
-      ($error (format nil "Argument ~a to `~a' is not a graph:" ar m) gr)))
+  (unless (or (graph-p gr) (digraph-p gr))
+    ($error (format nil "Argument ~a to `~a' is not a graph:" ar m) gr)))
 
 (defun $print_graph (gr)
   (require-graph-or-digraph 'print_graph 1 gr)
@@ -74,8 +76,8 @@
     ((graph-p gr)
      (format t "~%Graph on ~d vertices with ~d edges."
 	     (graph-size gr) (graph-order gr))
-     (if (> (graph-size gr) 0 )
-	 (format t "~%Adjacencies:"))
+     (when (> (graph-size gr) 0 )
+       (format t "~%Adjacencies:"))
      (dolist (v (graph-vertices gr))
        (format t "~% ~2d :" v)
        (dolist (u (neighbors v gr))
@@ -83,8 +85,8 @@
     (t
      (format t "~%Digraph on ~d vertices with ~d arcs."
 	     (digraph-size gr) (digraph-order gr))
-     (if (> (digraph-size gr) 0 )
-	 (format t "~%Adjacencies:"))
+     (when (> (digraph-size gr) 0 )
+       (format t "~%Adjacencies:"))
      (dolist (v (digraph-vertices gr))
        (format t "~% ~2d :" v)
        (dolist (u (out-neighbors v gr))
@@ -137,6 +139,18 @@
 	    (if w (set-edge-weight e w g))))
 	g) ))
 
+(defun $get_positions (gr)
+  (require-graph-or-digraph 'get_positions 1 gr)
+  (if (graph-p gr)
+      (graph-vertex-positions gr)
+      (digraph-vertex-positions gr)))
+
+(defun $set_positions (pos gr)
+  (require-graph-or-digraph 'set_positions 2 gr)
+  (if (graph-p gr)
+      (setf (graph-vertex-positions gr) pos)
+      (setf (digraph-vertex-positions gr) pos)))
+
 (defun $new_graph ()
   (make-graph))
 
@@ -155,8 +169,8 @@
   (if (graph-p gr) (graph-vertices gr) (digraph-vertices gr)))
 
 (defun require-vertex (m i v)
-  (if (not (integerp v))
-      ($error "Argument" i "to" m "is not a valid vertex.")))
+  (unless (integerp v)
+    ($error "Argument" i "to" m "is not a valid vertex.")))
 
 (defun is-vertex-in-graph (i gr)
   (not (equal (gethash i 
@@ -167,8 +181,8 @@
 	      'not-in-graph)))
 
 (defun require-vertex-in-graph (m i gr)
-  (if (not (is-vertex-in-graph i gr))
-      ($error m ": vertex not in graph.")))
+  (unless (is-vertex-in-graph i gr)
+    ($error m ": vertex not in graph.")))
 
 (defun $is_vertex_in_graph (i gr)
   (require-vertex 'vertex_in_graph 1 i)
@@ -178,15 +192,15 @@
 (defun $add_vertex (i gr)
   (require-vertex 'add_vertex 1 i)
   (require-graph-or-digraph 'add_vertex 2 gr)
-  (if (is-vertex-in-graph i gr)
-      ($error "add_vertex: vertex is already in the graph!"))
+  (when (is-vertex-in-graph i gr)
+    ($error "add_vertex: vertex is already in the graph!"))
   (add-vertex i gr)
   '$done)
 
 (defun $add_vertices (vl gr)
   (require-graph-or-digraph 'add_vertices 2 gr)
-  (if (not ($listp vl))
-      ($error "Argument 1 to add_vertices is not a list."))
+  (when (not ($listp vl))
+    ($error "Argument 1 to add_vertices is not a list."))
   (dolist (v (cdr vl))
     ($add_vertex v gr))
   '$done)
@@ -196,18 +210,19 @@
       (progn
 	(incf (graph-size gr))
 	(push i (graph-vertices gr))
+	(setf (graph-vertex-positions gr) nil)
 	(setf (gethash i (graph-neighbors gr)) ()))
       (progn
 	(incf (digraph-size gr))
 	(push i (digraph-vertices gr))
+	(setf (digraph-vertex-positions gr) nil)
 	(setf (gethash i (digraph-in-neighbors gr)) ())
-	(setf (gethash i (digraph-out-neighbors gr)) ()))
-      ))
+	(setf (gethash i (digraph-out-neighbors gr)) ())) ))
 
 (defun neighbors (i gr)
   (gethash i (if (graph-p gr)
 		 (graph-neighbors gr)
-		 (digraph-out-neighbors gr))))
+		 (digraph-out-neighbors gr))) )
 
 (defun in-neighbors (i gr)
   (gethash i (digraph-in-neighbors gr)))
@@ -250,8 +265,8 @@
 
 (defun $remove_vertices (vl gr)
   (require-graph-or-digraph 'remove_vertices 2 gr)
-  (if (not ($listp vl))
-      ($error "Argument 1 to remove_vertices is not a list."))
+  (when (not ($listp vl))
+    ($error "Argument 1 to remove_vertices is not a list."))
   (dolist (v (cdr vl))
     ($remove_vertex v gr))
   '$done)
@@ -263,6 +278,7 @@
 	  (let ((e (list (min u v) (max u v))))
 	    (remove-edge e gr)))
 	(clear-vertex-label v gr)
+	(setf (graph-vertex-positions gr) nil)
 	(setf (graph-vertices gr) (remove v (graph-vertices gr) :count 1))
 	(remhash v (graph-neighbors gr))
 	(decf (graph-size gr)))
@@ -272,6 +288,7 @@
 	(dolist (u (in-neighbors v gr))
 	  (remove-edge (list u v) gr))
 	(clear-vertex-label v gr)
+	(setf (digraph-vertex-positions gr) nil)
 	(setf (digraph-vertices gr) (remove v (digraph-vertices gr) :count 1))
 	(remhash v (digraph-in-neighbors gr))
 	(remhash v (digraph-out-neighbors gr))
@@ -293,10 +310,9 @@
     (t
      (let* ((v (first (graph-vertices gr))) (d (length (neighbors v gr))))
        (dolist (u (graph-vertices gr))
-	 (if (> (length (neighbors u gr)) d)
-	     (progn
-	       (setq d (length (neighbors u gr)))
-	       (setq v u))))
+	 (when (> (length (neighbors u gr)) d)
+	   (setq d (length (neighbors u gr)))
+	   (setq v u)))
        `((mlist simp) ,d ,v)))))
 
 (defun $min_degree (gr)
@@ -307,10 +323,9 @@
     (t
      (let* ((v (first (graph-vertices gr))) (d (length (neighbors v gr))))
        (dolist (u (graph-vertices gr))
-	 (if (< (length (neighbors u gr)) d)
-	     (progn
-	       (setq d (length (neighbors u gr)))
-	       (setq v u))))
+	 (when (< (length (neighbors u gr)) d)
+	   (setq d (length (neighbors u gr)))
+	   (setq v u)))
        `((mlist simp) ,d ,v)))))
 
 (defun $average_degree (gr)
@@ -382,14 +397,14 @@
     ((not (and ($listp e) (eq 2 ($length e))))
      ($error "Argument" ar "to" m "is not an edge (0)."))
     (t (let ((u ($first e)) (v ($second e)))
-	 (if (not (and (integerp u) (integerp v)))
-	     ($error "Argument" ar "to" m "is not an edge (1)."))
-	 (if (eq u v)
-	     ($error "Argument" ar "to" m "is not an edge (2)."))))))
+	 (unless (and (integerp u) (integerp v))
+	   ($error "Argument" ar "to" m "is not an edge (1)."))
+	 (when (eq u v)
+	   ($error "Argument" ar "to" m "is not an edge (2)."))))))
 
 (defun require-edge-in-graph (m e gr)
-  (if (not (is-edge-in-graph e gr))
-      ($error m ": edge not in graph.")))
+  (unless (is-edge-in-graph e gr)
+    ($error m ": edge not in graph.")))
 
 (defun m-edge-to-l-edge (e)
   (let ((uv (cdr e)))
@@ -417,7 +432,9 @@
 (defun $add_edge (e gr)
   (require-medge 'add_edge 1 e)
   (require-graph-or-digraph 'add_edge 2 gr)
-  (let* ((e1 (if (graph-p gr) (m-edge-to-l-edge e) (m-edge-to-l-dedge e)))
+  (let* ((e1 (if (graph-p gr)
+		 (m-edge-to-l-edge e)
+		 (m-edge-to-l-dedge e)))
 	 (u (first e1)) (v (second e1)))
     (cond
       ((not (and (is-vertex-in-graph u gr) (is-vertex-in-graph v gr)))
@@ -452,29 +469,35 @@
 	  (incf (digraph-order gr))))
     '$done))
 
+(defun add-edges (elist gr)
+  (dolist (e elist)
+    (add-edge e gr)))
+
 (defun $edges (gr)
   (require-graph-or-digraph 'edges 1 gr)
-  (let ((e (mapcar #'(lambda (u) `((mlist simp) ,@(copy-list u))) (edges gr))))
+  (let ((e (mapcar #'(lambda (u) `((mlist simp) ,@(copy-list u)))
+		   (edges gr))))
     `((mlist simp) ,@e)))
 
 (defun edges (gr)
-  (if (graph-p gr) (graph-edges gr) (digraph-edges gr)))
+  (if (graph-p gr)
+      (graph-edges gr)
+      (digraph-edges gr)))
 
 (defun $remove_edge (e gr)
   (require-medge 'remove_edge 1 e)
   (require-graph-or-digraph 'remove_edge 2 gr)
-  (if (not ($is_edge_in_graph e gr))
-      ($error "remove_edge: edge" e "is not in graph."))
-  (remove-edge
-   (if (graph-p gr)
-       (m-edge-to-l-edge e)
-       (m-edge-to-l-dedge e))
-   gr))
+  (unless ($is_edge_in_graph e gr)
+    ($error "remove_edge: edge" e "is not in graph."))
+  (remove-edge (if (graph-p gr)
+		   (m-edge-to-l-edge e)
+		   (m-edge-to-l-dedge e))
+	       gr))
 
 (defun $remove_edges (el gr)
   (require-graph-or-digraph 'remove_edges 2 gr)
-  (if (not ($listp el))
-      ($error "Argument 1 to remove_edges is not a list."))
+  (unless ($listp el)
+    ($error "Argument 1 to remove_edges is not a list."))
   (dolist (e (cdr el))
     ($remove_edge e gr))
   '$done)
@@ -507,17 +530,17 @@
   (require-graph 'contract_edge 2 gr)
   (let* ((e1 (m-edge-to-l-edge e)) (u (first e1)) (v (second e1)))
     (dolist (x (neighbors v gr))
-      (if (not (= x u))
-	  (let ((e2 (list (min x u) (max x u))))
-	    (if (not (is-edge-in-graph e2 gr))
-		(add-edge e2 gr)))))
+      (unless (= x u)
+	(let ((e2 (list (min x u) (max x u))))
+	  (unless (is-edge-in-graph e2 gr)
+	    (add-edge e2 gr)))))
     (remove-vertex v gr))
   '$done)
 
 (defun $contract_edges (el gr)
   (require-graph-or-digraph 'contract_edges 2 gr)
-  (if (not ($listp el))
-      ($error "Argument 1 to contract_edges is not a list."))
+  (unless ($listp el)
+    ($error "Argument 1 to contract_edges is not a list."))
   (dolist (e (cdr el))
     ($contract_edge e gr))
   '$done)
@@ -525,28 +548,27 @@
 (defun $get_edge_weight (e gr &optional default)
   (require-medge 'get_edge_weight 1 e)
   (require-graph-or-digraph 'get_edge_weight 2 gr)
-  (if (not ($is_edge_in_graph e gr))
-      (if (null default)
-	  ($error "get_edge_weight: edge not in graph")
-	  (return-from $get_edge_weight default)))
+  (unless ($is_edge_in_graph e gr)
+    (if (null default)
+	($error "get_edge_weight: edge not in graph")
+	(return-from $get_edge_weight default)))
   (let ((w (if (graph-p gr)
 	       (get-edge-weight (m-edge-to-l-edge e) gr)
 	       (get-edge-weight (m-edge-to-l-dedge e) gr))))
     (or w 1)))
 
 (defun get-edge-weight (e gr)
-  (let*
-      ((edge-weights
-	(if (graph-p gr)
-	    (graph-edge-weights gr)
-	    (digraph-edge-weights gr))))
+  (let* ((edge-weights
+	  (if (graph-p gr)
+	      (graph-edge-weights gr)
+	      (digraph-edge-weights gr))))
     (gethash e edge-weights)))
 
 (defun $clear_edge_weight (e gr)
   (require-medge 'clear_edge_weight 1 e)
   (require-graph-or-digraph 'clear_edge_weight 2 gr)
-  (if (not ($is_edge_in_graph e gr))
-      ($error "clear_edge_weight: edge not in graph"))
+  (unless ($is_edge_in_graph e gr)
+    ($error "clear_edge_weight: edge not in graph"))
   (if (graph-p gr)
       (clear-edge-weight (m-edge-to-l-edge e) gr)
       (clear-edge-weight (m-edge-to-l-dedge e) gr)))
@@ -563,8 +585,8 @@
 (defun $set_edge_weight (e w gr)
   (require-medge 'set_edge_weight 1 e)
   (require-graph-or-digraph 'set_edge_weight 3 gr)
-  (if (not ($is_edge_in_graph e gr))
-      ($error "set_edge_weight: edge not in graph"))
+  (unless ($is_edge_in_graph e gr)
+    ($error "set_edge_weight: edge not in graph"))
   (if (graph-p gr)
       (set-edge-weight (m-edge-to-l-edge e) w gr)
       (set-edge-weight (m-edge-to-l-dedge e) w gr)))
@@ -591,6 +613,17 @@
       ($add_edge `((mlist simp) ,u ,v) gr)))
   '$done)
 
+(defun $subdivide_edge (e gr)
+  (require-graph 'subdivide_edge 2 gr)
+  (require-edge-in-graph 'subdivide_edge (m-edge-to-l-edge e) gr)
+  (let ((new-vertex (1+ (apply #'max (vertices gr))))
+	(x ($first e))
+	(y ($second e)))
+    ($remove_edge (cons '(mlist simp) (list x y)) gr)
+    ($add_vertex new-vertex gr)
+    ($add_edge (cons '(mlist simp) (list x new-vertex)) gr)
+    ($add_edge (cons '(mlist simp) (list y new-vertex)) gr))
+  '$done)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;
@@ -623,7 +656,7 @@
   (let (elts)
     (maphash #'(lambda (key val)
 		 (declare (ignore val))
-		 (setq elts (cons key elts)))
+		 (push key elts))
 	     (ht-set-content set))
     elts))
 
@@ -647,10 +680,10 @@
     gr))
 
 (defun $create_graph (v_list e_list &optional dir)
-  (if (not (or (integerp v_list) ($listp v_list)))
-      ($error "Argument 1 to create_graph is not a list."))
-  (if (not ($listp e_list))
-      ($error "Argument 2 to create_graph is not a list."))
+  (unless (or (integerp v_list) ($listp v_list))
+    ($error "Argument 1 to create_graph is not a list."))
+  (unless ($listp e_list)
+    ($error "Argument 2 to create_graph is not a list."))
   (let ((gr (if dir (make-digraph) (make-graph))))
     (if (integerp v_list)
 	(dotimes (v v_list)
@@ -698,9 +731,10 @@
 (defun $petersen_graph (&optional n d)
   (if (null d)
       (setq n 5 d 2)
-      (if (not (and (integerp n) (integerp d)))
-	  ($error "Arguments to petersen_graph are not integers!")))
-  (let ((g ($empty_graph (* 2 n))))
+      (unless (and (integerp n) (integerp d))
+	($error "Arguments to petersen_graph are not integers!")))
+  (let ((g ($empty_graph (* 2 n)))
+	(positions ()))
     (dotimes (i n)
       (add-edge `(,i ,(+ n i)) g)
       (let* ((u (+ i n)) (v (+ (mod (+ i d) n) n))
@@ -708,30 +742,18 @@
 	(add-edge `(,e1 ,e2) g))
       (let* ((u (mod (1+ i) n)) (e1 (min u i)) (e2 (max u i)))
 	(add-edge `(,e1 ,e2) g)))
+    (dotimes (i n)
+      (push `((mlist simp) ,i ((mlist simp)
+			       ,($sin (/ (* 2 i pi) n))
+			       ,($cos (/ (* 2 i pi) n))))
+	    positions)
+      (push `((mlist simp) ,(+ n i) ((mlist simp)
+				     ,(* 0.66 ($sin (/ (* 2 i pi) n)))
+				     ,(* 0.66 ($cos (/ (* 2 i pi) n)))))
+	    positions))
+    ($set_positions (cons '(mlist simp) positions) g)
     g))
 
-(defun $dodecahedron_graph ()
-  (let
-      ((edg '((0 1) (1 2) (2 3) (3 4) (0 4) (0 15) (1 19)
-	      (2 18) (3 17) (4 16) (10 15) (10 16) (11 16)
-	      (11 17) (12 17) (12 18) (13 18) (13 19) (14 15)
-	      (14 19) (6 10) (8 11) (9 12) (7 13) (5 14) (5 6)
-	      (6 8) (8 9) (7 9) (5 7)))
-       (gr ($empty_graph 20)))
-    (dolist (e edg)
-      (add-edge e gr))
-    gr))
-
-(defun $heawood_graph ()
-  (let
-      ((edg '((0 1) (1 2) (2 3) (3 4) (4 5) (5 6) (6 7) (7 8)
-	      (8 9) (9 10) (10 11) (11 12) (12 13) (0 13)
-	      (0 9) (1 6) (2 11) (3 8) (4 13) (5 10) (7 12)))
-       (gr ($empty_graph 14)))
-    (dolist (e edg)
-      (add-edge e gr))
-    gr))
-
 (defun $complement_graph (gr)
   (require-graph 'complement_graph 1 gr)
   (let*
@@ -741,8 +763,9 @@
       (add-vertex v co))
     (dolist (u vrt)
       (dolist (v vrt)
-	(if (and (< u v) (not (is-edge-in-graph `(,u ,v) gr)))
-	    (add-edge `(,u ,v) co))))
+	(when (and (< u v) (not (is-edge-in-graph `(,u ,v) gr)))
+	  (add-edge `(,u ,v) co))))
+    (setf (graph-vertex-positions co) (graph-vertex-positions gr))
     co))
 
 (defun $complete_graph (n)
@@ -826,9 +849,8 @@
 	  (get-canonical-names (graph-edges gr))) (n (graph-order gr))
 	 (g ($empty_graph n)))
     (dotimes (i n)
-      (do
-       ((j (1+ i) (1+ j)))
-       ((= j n))
+      (do ((j (1+ i) (1+ j)))
+	  ((= j n))
 	(let ((e (car (rassoc i edge-list))) (f (car (rassoc j edge-list))))
 	  (if
 	   (or (member (first e) f) (member (second e) f))
@@ -838,64 +860,77 @@
 (defun $random_graph (n p)
   (if (not (integerp n))
       ($error "Argument 1 to random_graph is not an integer"))
-  (if (not (floatp p))
+  (if (not (floatp ($float p)))
       ($error "Argument 2 to random_graph is not a float"))
-  (let ((g ($empty_graph n)))
+  (let ((g ($empty_graph n))
+	(p ($float p)))
     (dotimes (i n)
       (do ((j (1+ i) (1+ j)))
 	  ((= j n))
-	(if (< (/ (random 10000) 10000.0) p)
+	(if (< (random 1.0) p)
 	    (add-edge `(,i ,j) g))))
     g))
 
 (defun $random_graph1 (n m)
-  (if (not (integerp n))
-      ($error "Argument 1 to random_graph is not an integer"))
-  (if (not (integerp m))
-      ($error "Argument 2 to random_graph is not an integer"))
-  (if (< (* n (1- n)) (* 2 m))
-      ($error "random_graph1: no such graph"))
-  (if (< (* n (1- n)) (* 4 m))
-      (return-from $random_graph1
-	($complement_graph ($random_graph1 n (- (/ (* n (1- n)) 2) m)))))
+  (unless (integerp n)
+    ($error "Argument 1 to random_graph is not an integer"))
+  (unless (integerp m)
+    ($error "Argument 2 to random_graph is not an integer"))
+  (when (< (* n (1- n)) (* 2 m))
+    ($error "random_graph1: no such graph"))
+  (when (< (* n (1- n)) (* 4 m))
+    (return-from $random_graph1
+      ($complement_graph ($random_graph1 n (- (/ (* n (1- n)) 2) m)))))
   (let ((g ($empty_graph n)))
     (do ((i 0)) ((= i m))
       (let ((u (random n)) (v (random n)))
-	(if (not (= u v))
-	    (let ((e (list (min u v) (max u v))))
-	      (if (not (is-edge-in-graph e g))
-		  (progn
-		    (setq i (1+ i))
-		    (add-edge e g)))))))
+	(unless (= u v)
+	  (let ((e (list (min u v) (max u v))))
+	    (unless (is-edge-in-graph e g)
+	      (setq i (1+ i))
+	      (add-edge e g))))))
+    g))
+
+(defun $random_bipartite_graph (a b p)
+  (unless (integerp a)
+    ($error "Argument 1 to random graph is not an integer"))
+  (unless (integerp b)
+    ($error "Argument b to random graph is not an integer"))
+  (let ((g ($empty_graph (+ a b))))
+    (dotimes (x a)
+      (dotimes (y b)
+	(when (< (random 1.0) p)
+	  (add-edge (list x (+ a y)) g))))
     g))
 
 (defun $random_digraph (n p)
-  (if (not (integerp n))
-      ($error "Argument 1 to random_digraph is not an integer"))
-  (if (not (floatp p))
-      ($error "Argument 2 to random_digraph is not a float"))
-  (let ((g ($empty_digraph n)))
+  (unless (integerp n)
+    ($error "Argument 1 to random_digraph is not an integer"))
+  (unless (floatp ($float p))
+    ($error "Argument 2 to random_digraph is not a float"))
+  (let ((g ($empty_digraph n))
+	(p ($float p)))
     (dotimes (i n)
       (dotimes (j n)
-	(if (and (not (= i j)) (< (/ (random 10000) 10000.0) p))
-	    (add-edge `(,i ,j) g))))
+	(when (and (not (= i j)) (< (random 1.0) p))
+	  (add-edge `(,i ,j) g))))
     g))
 
 (defun $random_tournament (n)
-  (if (not (integerp n))
-      ($error "Argument 1 to random_tournament is not an integer"))
+  (unless (integerp n)
+    ($error "Argument 1 to random_tournament is not an integer"))
   (let ((g ($empty_digraph n)))
     (dotimes (i n)
       (do ((j (1+ i) (1+ j)))
 	  ((= j n))
-	(if (and (not (= i j)) (< (/ (random 10000) 10000.0) 0.5))
+	(if (and (not (= i j)) (< (random 1.0) 0.5))
 	    (add-edge `(,i ,j) g)
 	    (add-edge `(,j ,i) g))))
     g))
 
 (defun $random_tree (n)
-  (if (or (not (integerp n)) (<= n 0))
-      ($error "Argument 1 to random_tree is not a positive integer"))
+  (when (or (not (integerp n)) (<= n 0))
+    ($error "Argument 1 to random_tree is not a positive integer"))
   (let*
       ((tr ($empty_graph n))
        (vrt (remove 0 (graph-vertices tr) :count 1))
@@ -912,60 +947,99 @@
 (defun $underlying_graph (gr)
   (require-digraph 'underlying_graph 1 gr)
   (let ((g (make-graph)))
-    (dolist (v (graph-vertices gr))
+    (dolist (v (vertices gr))
       (add-vertex v g))
-    (dolist (e (graph-edges gr))
+    (dolist (e (digraph-edges gr))
       (let ((u (first e)) (v (second e)))
 	(let ((e1 (list (apply #'min e) (apply #'max e))))
-	  (if (not (is-edge-in-graph e1 g))
-	      (add-edge `(,u ,v) g)))))
+	  (when (not (is-edge-in-graph e1 g))
+	    (add-edge `(,u ,v) g)))))
     g))
 
 (defun $induced_subgraph (vl gr)
   (require-graph 2 'induced_subgraph gr)
-  (if (not ($listp vl))
-      ($error "First argument to induced_subgraph is not a list."))
+  (unless ($listp vl)
+    ($error "First argument to induced_subgraph is not a list."))
   (let
       ((v_l (cdr vl))
        (g (make-graph)))
     (dolist (v v_l)
-      (if (not (is-vertex-in-graph v gr))
-	  ($error
-	   "induced_subgraph: second argument is not a list of vertices"))
+      (when (not (is-vertex-in-graph v gr))
+	($error
+	 "induced_subgraph: second argument is not a list of vertices"))
       (add-vertex v g)
       (let ((l (get-vertex-label v gr)))
-	(if l (set-vertex-label v l g))))
+	(when l
+	  (set-vertex-label v l g))))
     (dolist (e (graph-edges gr))
       (let ((u (first e)) (v (second e)))
-	(if (and (member u v_l) (member v v_l))
-	    (add-edge e g))))
+	(when (and (member u v_l) (member v v_l))
+	  (add-edge e g))))
     g))
 
 (defun $wheel_graph (n)
-  (if (not (and (integerp n) (>= n 3)))
-      ($error "wheel_graph: first argument is no an integer greater than 3"))
-  (let ((g ($cycle_graph n)))
+  (unless (and (integerp n) (>= n 3))
+    ($error "wheel_graph: first argument is no an integer greater than 3"))
+  (let ((g ($cycle_graph n))
+	(positions ()))
     (add-vertex n g)
     (dotimes (i n)
       (add-edge `(,i ,n) g))
+    (dotimes (i n)
+      (push `((mlist simp) ,i ((mlist simp)
+			       ,($sin (/ (* 2 i pi) n))
+			       ,($cos (/ (* 2 i pi) n))))
+	    positions))
+    (push `((mlist simp) ,n ((mlist simp) 0 0)) positions)
+    ($set_positions (cons '(mlist simp) positions) g)
     g))
 
 (defun $circulant_graph (n l)
-  (if (not (and (integerp n) (> n 0)))
-      ($error "Argument 1 to circulant_graph is not a positive integer."))
-  (if (not ($listp l))
-      ($error "Argument 2 to circulant_graph is not a list."))
-  (let ((g ($empty_graph n)))
+  (unless (and (integerp n) (> n 0))
+    ($error "Argument 1 to circulant_graph is not a positive integer."))
+  (unless ($listp l)
+    ($error "Argument 2 to circulant_graph is not a list."))
+  (let ((g ($empty_graph n))
+	(positions ()))
     (dolist (d (cdr l))
-      (if (not (and (integerp d) (> d 0)))
-	  ($error
-	   "Argument 2 to circulant graph is no a list of positive integers"))
+      (unless (and (integerp d) (> d 0))
+	($error
+	 "Argument 2 to circulant graph is no a list of positive integers"))
       (dotimes (i n)
 	(let ((e `(,i ,(mod (+ i d) n))))
 	  (setq e (list (apply #'min e) (apply #'max e)))
 	  (add-edge e g))))
+    (dotimes (i n)
+      (push `((mlist simp) ,i ((mlist simp)
+			       ,($sin (m// (m* 2.0 i '$%pi) n))
+			       ,($cos (m// (m* 2.0 i '$%pi) n))))
+	    positions))
+    ($set_positions (cons '(mlist simp) positions) g)
     g))
 
+(defun $flower_snark (&optional (o 5))
+  (let ((g ($empty_graph (* 4 o))))
+    (labels
+	((make-edge (u v)
+	   (list (min u v) (max u v)))
+	 (a (i)
+	   (mod i o))
+	 (b (i)
+	   (+ o (mod i o)))
+	 (c (i)
+	   (+ o o (mod i o)))
+	 (d (i)
+	   (+ o o o (mod i o))))
+      (dotimes (i o)
+	(add-edge (make-edge (a i) (a (1+ i))) g)
+	(add-edge (make-edge (a i) (b i)) g)
+	(add-edge (make-edge (b i) (c i)) g)
+	(add-edge (make-edge (b i) (d i)) g)
+	(add-edge (make-edge (c i) (d (1+ i))) g)
+	(add-edge (make-edge (d i) (c (1+ i))) g) ))
+    g))
+
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; graph properties
@@ -979,28 +1053,26 @@
 
 (defun $connected_components (gr)
   (require-graph 'connected_components 1 gr)
-  (if (= 0 (graph-size gr))
-      (return-from $connected_components '((mlist simp))))
-  (let ((components ()) (visited ()))
-    (do ((vrt (copy-list (graph-vertices gr))))
-	((= (length vrt) 0))
-      (let ((c ()) (v (first vrt)) (active ()))
-	(push v active)
-	(do ()
-	    ((null active))
-	  (let ((x (pop active)))
-	    (push x c)
-	    (push x visited)
-	    (setq vrt (remove x vrt :count 1))
-	    (dolist (u (neighbors x gr))
-	      (if (not (or (member u visited) (member u active)))
-		  (push u active)))))
-	(push `((mlist simp) ,@c) components)))
+  (when (= 0 (graph-size gr))
+    (return-from $connected_components '((mlist simp))))
+  (let ((components ()) (visited (make-hash-table)))
+    (loop for v in (vertices gr) do
+	 (unless (gethash v visited)
+	   (let ((c ()) (active ()))
+	     (push v active)
+	     (loop while active do
+		  (let ((x (pop active)))
+		    (push x c)
+		    (setf (gethash x visited) t)
+		    (dolist (u (neighbors x gr))
+		      (unless (or (gethash u visited) (member u active))
+			(push u active)))))
+	     (push `((mlist simp) ,@c) components))))
     `((mlist simp) ,@components)))
 
 (defun $is_connected (gr)
   (require-graph 'is_connected 1 gr)
-  (eq ($length ($connected_components gr)) 1))
+  (= ($length ($connected_components gr)) 1))
 
 (defun $is_tree (gr)
   (require-graph 'is_tree 1 gr)
@@ -1012,14 +1084,16 @@
 ;;;
 
 (defun $adjacency_matrix (gr)
-  (require-graph 'adjacency_matrix 1 gr)
-  (let ((m ($zeromatrix (graph-size gr) (graph-size gr)))
-	(names (get-canonical-names (vertices gr))))
-    (dolist (e (graph-edges gr))
+  (require-graph-or-digraph 'adjacency_matrix 1 gr)
+  (let* ((n (if (graph-p gr) (graph-size gr) (digraph-size gr)))
+	 (m ($zeromatrix n n))
+	 (names (get-canonical-names (vertices gr))))
+    (dolist (e (edges gr))
       (setf (nth (1+ (cdr (assoc (first e) names)))
 		 (nth (1+ (cdr (assoc (second e) names))) m)) 1)
-      (setf (nth (1+ (cdr (assoc (second e) names)))
-		 (nth (1+ (cdr (assoc (first e) names))) m)) 1))
+      (when (graph-p gr)
+	(setf (nth (1+ (cdr (assoc (second e) names)))
+		   (nth (1+ (cdr (assoc (first e) names))) m)) 1)))
     m))
 
 (defun $laplacian_matrix (gr)
@@ -1095,12 +1169,17 @@
 ;;; diameter, radius
 ;;;
 
-(defun $diameter (gr)
-  (require-graph 'diameter 1 gr)
-  (if (not ($is_connected gr))
-      ($error "diameter: graph is not connected."))
-  (let ((diameter 0))
-    (dolist (v (graph-vertices gr))
+(defun $vertex_eccentricity (v gr)
+  (require-graph 'vertex_eccentricity 1 gr)
+  (require-vertex-in-graph 'vertex_eccentricity v gr)
+  (let ((ecc (eccentricity (list v) gr)))
+    (gethash v ecc)))
+
+(defun eccentricity (v_list gr)
+  (unless ($is_connected gr)
+    ($error "eccentricity: graph is not connected."))
+  (let ((ecc (make-hash-table)))
+    (dolist (v v_list)
       (let
 	  ((visited (new-set v))
 	   (active (new-set v))
@@ -1116,36 +1195,63 @@
 		(set-add w next))))
 	  (setq active next)
 	  (setq depth (1+ depth)))
-	(if (> depth diameter)
-	    (setq diameter depth))))
+	(setf (gethash v ecc) depth)))
+      ecc))
+
+(defun $diameter (gr)
+  (require-graph 'diameter 1 gr)
+  (let ((ecc (eccentricity (vertices gr) gr))
+	(diameter 0))
+    (maphash #'(lambda (key val)
+		 (declare (ignore key))
+		 (when (> val diameter)
+		   (setq diameter val)))
+	     ecc)
     diameter))
 
 (defun $radius (gr)
   (require-graph 'radius 1 gr)
-  (if (not ($is_connected gr))
-      ($error "radius: graph is not connected."))
-  (let ((radius (graph-size gr)))
-    (dolist (v (graph-vertices gr))
-      (let
-	  ((visited (new-set v))
-	   (active (new-set v))
-	   (next)
-	   (depth -1))
-	(do ()
-	    ((set-emptyp active))
-	  (setq next (new-set))
-	  (dolist (u (set-elements active))
-	    (dolist (w (neighbors u gr))
-	      (if (not (set-member w visited))
-		  (progn
-		    (set-add w visited)
-		    (set-add w next)))))
-	  (setq active next)
-	  (setq depth (1+ depth)))
-	(if (< depth radius)
-	    (setq radius depth))))
+  (let ((ecc (eccentricity (vertices gr) gr))
+	(radius ($graph_size gr)))
+    (maphash #'(lambda (key val)
+		 (declare (ignore key))
+		 (when (< val radius)
+		   (setq radius val)))
+	     ecc)
     radius))
 
+(defun $graph_center (gr)
+  (require-graph 'graph_center 1 gr)
+  (let ((ecc (eccentricity (vertices gr) gr))
+	(per ())
+	(radius ($graph_size gr)))
+    (maphash #'(lambda (key val)
+		 (declare (ignore key))
+		 (when (< val radius)
+		   (setq radius val)))
+	     ecc)
+    (maphash #'(lambda (key val)
+		 (when (= val radius)
+		   (push key per)))
+	     ecc)
+    `((mlist simp) ,@per)))
+
+(defun $graph_periphery (gr)
+  (require-graph 'graph_periphery 1 gr)
+  (let ((ecc (eccentricity (vertices gr) gr))
+	(center ())
+	(diameter 0))
+    (maphash #'(lambda (key val)
+		 (declare (ignore key))
+		 (when (> val diameter)
+		   (setq diameter val)))
+	     ecc)
+    (maphash #'(lambda (key val)
+		 (when (= val diameter)
+		   (push key center)))
+	     ecc)
+    `((mlist simp) ,@center)))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; bipartition
@@ -1153,8 +1259,8 @@
 
 (defun $bipartition (gr)
   (require-graph 'bipartition 1 gr)
-  (if (= (graph-size gr) 0)
-      (return-from $bipartition `((mlist simp))))
+  (when (= (graph-size gr) 0)
+    (return-from $bipartition `((mlist simp))))
   (let ((components (cdr ($connected_components gr))) (A ()) (B ()))
     (dolist (c components)
       (let ((partition (bi-partition (first (cdr c)) gr)))
@@ -1184,12 +1290,11 @@
 	    (push w B))
 	(dolist (u (neighbors w gr))
 	  (if (set-member u visited)
-	      (if (= (gethash u colors) wc)
-		  (return-from bi-partition ()))
-	      (if (not (member u active))
-		  (progn
-		    (push u active)
-		    (setf (gethash u colors) (- 1 wc))))))))
+	      (when (= (gethash u colors) wc)
+		(return-from bi-partition ()))
+	      (unless (member u active)
+		(push u active)
+		(setf (gethash u colors) (- 1 wc)))))))
     `(,A ,B)))
 
 (defun $is_bipartite (gr)
@@ -1238,10 +1343,10 @@
       (let ((curr-comp ()))
 	(dolist (e c)
 	  (let ((u (first e)) (v (second e)))
-	    (if (not (member u curr-comp))
-		(push u curr-comp))
-	    (if (not (member v curr-comp))
-		(push v curr-comp))))
+	    (unless (member u curr-comp)
+	      (push u curr-comp))
+	    (unless (member v curr-comp)
+	      (push v curr-comp))))
 	(setq bicomp (cons `((mlist simp) ,@(sort curr-comp #'<)) bicomp))))
     `((mlist simp) ,@bicomp)))
 
@@ -1250,8 +1355,8 @@
   (setf (gethash w *dfs-bicomp-num*) *dfs-bicomp-depth*)
   (setf (gethash w *dfs-bicomp-low-pt*) *dfs-bicomp-depth*)
   (dolist (u (neighbors w gr))
-    (if (< (gethash u *dfs-bicomp-num*) (gethash w *dfs-bicomp-num*))
-	(push `(,w ,u) *dfs-bicomp-edges*))
+    (when (< (gethash u *dfs-bicomp-num*) (gethash w *dfs-bicomp-num*))
+      (push `(,w ,u) *dfs-bicomp-edges*))
     (if (= 0 (gethash u *dfs-bicomp-num*))
 	(progn
 	  (dfs-bicomponents gr u)
@@ -1300,7 +1405,7 @@
 	      (setq *scon-depth* 0)
 	      (dfs-strong-components gr (first *scon-vrt*))
 	      (dolist (c *scon-comp*)
-		(setq res (cons c res)))
+		(push c res))
 	      (setq *scon-comp* ()))
 	`((mlist simp) ,@res))))
 
@@ -1309,27 +1414,57 @@
   (setf (gethash v *scon-dfn*) *scon-depth*)
   (setf (gethash v *scon-low*) *scon-depth*)
   (setf *scon-vrt* (remove v *scon-vrt* :count 1))
-  (setf *scon-st* (cons v *scon-st*))
+  (push v *scon-st*)
   (dolist (u (neighbors v gr))
     (if (gethash u *scon-dfn*)
-	(if (and (< (gethash u *scon-dfn*) (gethash v *scon-dfn*))
-		 (member u *scon-st*))
-	    (setf (gethash v *scon-low*)
-		  (min (gethash v *scon-low*)
-		       (gethash u *scon-dfn*))))
+	(when (and (< (gethash u *scon-dfn*) (gethash v *scon-dfn*))
+		   (member u *scon-st*))
+	  (setf (gethash v *scon-low*)
+		(min (gethash v *scon-low*)
+		     (gethash u *scon-dfn*))))
 	(progn
 	  (dfs-strong-components gr u)
 	  (setf (gethash v *scon-low*)
 		(min (gethash v *scon-low*)
 		     (gethash u *scon-low*))))))
-  (if (= (gethash v *scon-low*) (gethash v *scon-dfn*))
-      (let ((x (pop *scon-st*))
-	    (comp ()))
-	(loop while (not (= x v)) do
-	  (setq comp (cons x comp))
-	  (setq x (pop *scon-st*)))
-	(setf comp (cons x comp))
-	(setq *scon-comp* (cons `((mlist simp) ,@comp) *scon-comp*)))))
+  (when (= (gethash v *scon-low*) (gethash v *scon-dfn*))
+    (let ((x (pop *scon-st*))
+	  (comp ()))
+      (loop while (not (= x v)) do
+	   (push x comp)
+	   (setq x (pop *scon-st*)))
+      (push x comp)
+      (setq *scon-comp* (cons `((mlist simp) ,@comp) *scon-comp*)))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; topological sorting
+;;;
+
+(defun $topological_sort (dag)
+  (require-digraph 'topological_sort 1 dag)
+  (let ((in-degrees (make-hash-table))
+	(q ())
+	(n ($graph_order dag))
+	(s ()))
+    (dolist (v (vertices dag))
+      (setf (gethash v in-degrees) 0))
+    (dolist (e (edges dag))
+      (incf (gethash (second e) in-degrees)))
+    (dolist (v (vertices dag))
+      (when (= (gethash v in-degrees) 0)
+	(push v q)))
+    (loop while (> (length q) 0) do
+	 (let ((v (pop q)))
+	   (push v s)
+	   (dolist (u (out-neighbors v dag))
+	     (decf (gethash u in-degrees))
+	     (decf n)
+	     (when (= (gethash u in-degrees) 0)
+	       (push u q)))))
+    (if (= n 0)
+	`((mlist simp) ,@(reverse s))
+	'((mlist simp)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -1543,10 +1678,10 @@
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
-;;; maximal clique
+;;; maximum clique
 ;;;
 
-(defvar *maximal-clique* ())
+(defvar *maximum-clique* ())
 
 (defun vertices-by-degrees (gr)
   (let ((vrt ()))
@@ -1573,32 +1708,51 @@
 
 (defun $max_clique (gr)
   (require-graph 'max_clique 1 gr)
-  (setq *maximal-clique* ())
+  (setq *maximum-clique* ())
   (let ((v) (coloring) (h ($copy_graph gr)))
-    (do () ((= 0 (graph-size h)))
+    (do () ((= (length *maximum-clique*) (graph-size h)))
       (setq coloring (greedy-color h))
       (setq v ($second ($max_degree h)))
       (extend-clique `(,v) (neighbors v h) coloring h)
       (remove-vertex v h)))
-  `((mlist simp) ,@(sort *maximal-clique* #'<)))
+  `((mlist simp) ,@(sort *maximum-clique* #'<)))
 
 (defun $max_independent_set (gr)
   (require-graph 'max_independent_set 1 gr)
-  ($max_clique ($complement_graph gr)))
+  (if ($is_bipartite gr)
+      (let ((mis)
+	    (vc (cdr ($min_vertex_cover gr))))
+	(loop for v in (vertices gr) do
+	     (unless (member v vc)
+	       (setq mis (cons v mis))))
+	`((mlist simp) ,@mis))
+      ($max_clique ($complement_graph gr))))
+
+(defun $min_vertex_cover (gr)
+  (require-graph 'min_vertex_cover 1 gr)
+  (let ((bipart ($bipartition gr)))
+    (if (null (cdr bipart)) 
+	(let ((vc)
+	      (mc (cdr ($max_clique ($complement_graph gr)))))
+	  (loop for v in (vertices gr) do
+	       (unless (member v mc)
+		 (setq vc (cons v vc))))
+	  `((mlist simp) ,@vc))
+	(maximum-matching-bipartite gr (cadr bipart) (caddr bipart) t))))
 
 (defun extend-clique (clique neigh coloring gr)
   (if (= (length neigh) 0)
-      ;; did we get to a clique maximal clique
-      (if (> (length clique) (length *maximal-clique*))
+      ;; did we get to a maximal clique
+      (if (> (length clique) (length *maximum-clique*))
 	  (progn
-	    (setq *maximal-clique* clique)
+	    (setq *maximum-clique* clique)
 	    (return-from extend-clique)))
       ;; can we do better?
       (let ((colors ()))
 	(dolist (x neigh)
 	  (if (not (member (gethash x coloring) colors))
 	      (push (gethash x coloring) colors)))
-	(if (> (+ (length clique) (length colors)) (length *maximal-clique*))
+	(if (> (+ (length clique) (length colors)) (length *maximum-clique*))
 	    ;; try improving
 	    (do () ((= (length neigh) 0))
 	      (let* ((x (first neigh)) (new-clique (cons x clique))
@@ -1851,7 +2005,6 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (defun $temp_filename (file)
-  (unless ($stringp file)
+  (unless (stringp file)
     ($error "Argument to `temp_filename' is not a string"))
-  (plot-temp-file (maybe-invert-string-case (symbol-name (stripdollar file)))))
-
+  (plot-temp-file file))
