@@ -242,7 +242,7 @@
 		 (expon (caddr x)) ;; this is the exponent
 		 (doit (and
 			f		; there is such a function
-			(member (getchar f 1) '(% $) :test #'eq) ;; insist it is a % or $ function
+			(member (getcharn f 1) '(#\% #\$)) ;; insist it is a % or $ function
 			(not (eq (car (last (car fx))) 'array))	; fix for x[i]^2
 					; Jesper Harder <harder@ifa.au.dk>
 			(not (member f '(%sum %product %derivative %integrate %at
@@ -502,12 +502,6 @@
  
 (defprop | --> | " rightarrow " texsym)
 (defprop | WHERE | "` bold where`" texsym)
-(defprop &>= (" ge ") texsym)
-(defprop &> (" > ") texsym)
-(defprop &<= (" le ") texsym)
-(defprop &< (" < ") texsym)
-(defprop &= (" = ") texsym)
-(defprop |&#| (" neq ") texsym)
  
 (defun tex-mlable (x l r)
   (tex (caddr x)
