@@ -16,8 +16,8 @@
   "Convert a Maxima matrix A of dimension NROW and NCOL to Lisp matrix
   suitable for use with LAPACK"
   (let* ((array-type (if (complex-maxima-matrix-p a)
-			 '(complex double-float)
-			 'double-float))
+			 '(complex flonum)
+			 'flonum))
 	 (mat (make-array (* nrow ncol)
 			  :element-type array-type))
 	 (mat-2d (make-array (list ncol nrow)
@@ -28,10 +28,10 @@
       (let ((c 0))
 	(dolist (col (cdr row))
 	  ;; Fortran matrices are in column-major order!
-	  (setf (aref mat-2d c r) (if (eql array-type 'double-float)
-				      (coerce col 'double-float)
+	  (setf (aref mat-2d c r) (if (eql array-type 'flonum)
+				      (coerce col 'flonum)
 				      (coerce (complex ($realpart col) ($imagpart col))
-					      '(complex double-float))
+					      '(complex flonum))
 				      ))
 	  (incf c)))
       (incf r))
@@ -40,7 +40,7 @@
 (defun lapack-maxify-matrix (nrow ncol a)
   "Convert an LAPACK matrix of dimensions NROW and NCOL into a Maxima
 matrix (list of lists)"
-  (let ((2d (make-array (list ncol nrow) :element-type 'double-float
+  (let ((2d (make-array (list ncol nrow) :element-type 'flonum
 			:displaced-to a)))
     (let (res)
       (dotimes (r nrow)
@@ -112,12 +112,12 @@ eigenvectors."
     
     (let* ((n (maxima-matrix-dims a))
 	   (a-mat (lapack-lispify-matrix a n n))
-	   (wr (make-array n :element-type 'double-float))
-	   (wi (make-array n :element-type 'double-float))
+	   (wr (make-array n :element-type 'flonum))
+	   (wi (make-array n :element-type 'flonum))
 	   (vl (make-array (if left-vec-p (* n n) 0)
-			   :element-type 'double-float))
+			   :element-type 'flonum))
 	   (vr (make-array (if right-vec-p (* n n) 0)
-			   :element-type 'double-float)))
+			   :element-type 'flonum)))
       ;; XXX: FIXME: We need to do more error checking in the calls to
       ;; dgeev!
       (multiple-value-bind (z-jobvl z-jobvr z-n z-a z-lda z-wr z-wi z-vl
@@ -129,7 +129,7 @@ eigenvectors."
 	(declare (ignore z-jobvl z-jobvr z-n z-a z-lda z-wr z-wi z-vl
 			 z-ldvl z-vr z-ldvr z-work z-lwork info))
 	(let* ((opt-lwork (truncate (aref wr 0)))
-	       (work (make-array opt-lwork :element-type 'double-float)))
+	       (work (make-array opt-lwork :element-type 'flonum)))
 	  ;; Now do the work with the optimum size of the work space.
 	  (multiple-value-bind (z-jobvl z-jobvr z-n z-a z-lda z-wr z-wi z-vl
 					z-ldvl z-vr z-ldvr z-work z-lwork info)
@@ -138,6 +138,10 @@ eigenvectors."
 			    n a-mat n wr wi vl n vr n work opt-lwork 0)
 	    (declare (ignore z-jobvl z-jobvr z-n z-a z-lda z-wr z-wi z-vl
 			     z-ldvl z-vr z-ldvr z-work z-lwork))
+	    (cond ((< info 0)
+		   (merror "DGEEV: invalid arguments: ~D" info))
+		  ((> info 0)
+		   (merror "DGEEV: failed to converge: ~D" info)))
 	    ;; Convert wr+%i*wi to maxima form
 	    #+nil
 	    (progn
@@ -146,14 +150,14 @@ eigenvectors."
 	      (format t "vr = ~A~%" vr))
 	    (let ((e-val (make-eigval wr wi))
 		  (e-vec-right (if right-vec-p
-					 (list (make-eigvec n vr wi))
-					 nil))
+				   (make-eigvec n vr wi)
+				   nil))
 		  (e-vec-left (if left-vec-p
-					(list (make-eigvec n vl wi))
-					nil)))
-	      `((mlist) ,e-val ,@e-vec-right ,@e-vec-left))))))))
+				  (make-eigvec n vl wi)
+				  nil)))
+	      `((mlist) ,e-val ,e-vec-right ,e-vec-left))))))))
 
-(defun $dgesvd (a jobu jobvt)
+(defun $dgesvd (a &optional jobu jobvt)
   "
 DGESVD computes the singular value decomposition (SVD) of a real
 M-by-N matrix A, optionally computing the left and/or right singular
@@ -171,8 +175,9 @@ U and V are the left and right singular vectors of A.
 Note that the routine returns V**T, not V.
 
 A list of three items is returned.  The first is a list containing the
-non-zero elements of SIGMA.  The second is the matrix U.  The third is
-V**T."
+non-zero elements of SIGMA.  If jobu is not false, The second element
+is the matrix U.  Otherwise it is false.  Similarly, the third element
+is V**T or false, depending on jobvt."
   
   (flet ((maxify-vector (v)
 	   `((mlist) ,@(coerce v 'list)))
@@ -183,15 +188,15 @@ V**T."
     (multiple-value-bind (nrow ncol)
 	(maxima-matrix-dims a)  
       (let* ((a-mat (lapack-lispify-matrix a nrow ncol))
-	     (s (make-array (min nrow ncol) :element-type 'double-float))
-	     (u (make-array (* nrow nrow) :element-type 'double-float))
-	     (u1 (make-array (list nrow nrow) :element-type 'double-float
+	     (s (make-array (min nrow ncol) :element-type 'flonum))
+	     (u (make-array (* nrow nrow) :element-type 'flonum))
+	     (u1 (make-array (list nrow nrow) :element-type 'flonum
 			     :displaced-to u))
 	     (vt (make-array (* ncol ncol)
-			     :element-type 'double-float))
-	     (vt1 (make-array (list ncol ncol) :element-type 'double-float
+			     :element-type 'flonum))
+	     (vt1 (make-array (list ncol ncol) :element-type 'flonum
 			      :displaced-to vt))
-	     (wr (make-array 1 :element-type 'double-float)))
+	     (wr (make-array 1 :element-type 'flonum)))
 	;; XXX: FIXME: We need to do more error checking in the calls to
 	;; dgesvd!
 	(multiple-value-bind (z-jobu z-jobvt z-m z-n z-a z-lda z-s z-u z-ldu
@@ -207,7 +212,7 @@ V**T."
 	  (declare (ignore z-jobu z-jobvt z-m z-n z-a z-lda z-s z-u z-ldu
 			   z-vt z-ldvt z-work z-lwork info))
 	  (let* ((opt-lwork (truncate (aref wr 0)))
-		 (work (make-array opt-lwork :element-type 'double-float)))
+		 (work (make-array opt-lwork :element-type 'flonum)))
 	    ;; Allocate the optimum work array and do the requested
 	    ;; computation.
 	    (multiple-value-bind (z-jobu z-jobvt z-m z-n z-a z-lda z-s z-u
@@ -220,9 +225,17 @@ V**T."
 				work opt-lwork
 				0)
 	      (declare (ignore z-jobu z-jobvt z-m z-n z-a z-lda z-s z-u z-ldu
-			       z-vt z-ldvt z-work z-lwork info))
-	      (let ((u-max (lapack-maxify-matrix nrow nrow u1))
-		    (vt-max (lapack-maxify-matrix ncol ncol vt1))
+			       z-vt z-ldvt z-work z-lwork))
+	      (cond ((< info 0)
+		     (merror "DGESVD: invalid arguments: ~D" info))
+		    ((> info 0)
+		     (merror "DGESVD: failed to converge: ~D" info)))
+	      (let ((u-max (if jobu
+			       (lapack-maxify-matrix nrow nrow u1)
+			       nil))
+		    (vt-max (if jobvt
+				(lapack-maxify-matrix ncol ncol vt1)
+				nil))
 		    (s-max (maxify-vector s)))
 		`((mlist) ,s-max ,u-max ,vt-max)))))))))
 
@@ -255,7 +268,7 @@ squares).  Note that  max(abs(A(i,j)))  is not a  matrix norm."
 			($inf_norm "I")
 			($frobenius "F")))
 	   (work (make-array (if (equal norm-type "I") nrows 0)
-			     :element-type 'double-float)))
+			     :element-type 'flonum)))
       (lapack::dlange norm-type nrows ncols a-mat nrows work))))
 
 
@@ -286,6 +299,6 @@ squares).  Note that  max(abs(A(i,j)))  is not a  matrix norm."
 			($inf_norm "I")
 			($frobenius "F")))
 	   (work (make-array (if (equal norm-type "I") nrows 0)
-			     :element-type 'double-float)))
+			     :element-type 'flonum)))
       (lapack::zlange norm-type nrows ncols a-mat nrows work))))
       
