@@ -102,6 +102,7 @@
   (setq y (simpcheck (cadr x) z))
   (cond ((or (floatp y) (and (not makef) (ratnump y) (equal (caddr y) 2)))
 	 (simplifya (makegamma1 (list '(mfactorial) y)) nil))
+	(($bfloatp y) (mfuncall '$bffac y $fpprec))
 	((or (not (fixnump y)) (not (> y -1)))
 	 (eqtest (list '(mfactorial) y) x))
 	((or (minusp $factlim) (not (> y $factlim)))
@@ -393,6 +394,25 @@ summation when necessary."
 	      (getl '%sum '($outative $linear)))
 	 (freesum exp lo hi 1))
 	((null $simpsum) (list (get '%sum 'msimpind) exp i lo hi))
+	((and (or (eq lo '$minf)
+		  (alike1 lo '((mtimes simp) -1 $inf)))
+	      (equal hi '$inf))
+	 (let ((pos-part (simpsum2 exp i 0 '$inf))
+	       (neg-part (simpsum2 (maxima-substitute (m- i) i exp) i 1 '$inf)))
+	   (cond
+	     ((or (eq neg-part '$und)
+		  (eq pos-part '$und))
+	      '$und)
+	     ((eq pos-part '$inf)
+	      (if (eq neg-part '$minf) '$und '$inf))
+	     ((eq pos-part '$minf)
+	      (if (eq neg-part '$inf) '$und '$minf))
+	     ((or (eq neg-part '$inf) (eq neg-part '$minf))
+	      neg-part)
+	     (t (m+ neg-part pos-part)))))
+	((or (eq lo '$minf)
+	     (alike1 lo '((mtimes simp) -1 '$inf)))
+	 (simpsum2 (maxima-substitute (m- i) i exp) i (m- hi) '$inf))
 	(t (simpsum2 exp i lo hi))))
 
 ;; DOSUM, MEVALSUMARG, DO%SUM -- general principles
