@@ -239,7 +239,7 @@
   (setq x (cond ((null x)
 		 (ecase (array-element-type ar)
 		   (fixnum '(0))
-		   (float '(0d0))
+		   (float '(0.0))
 		   ((t) '(nil))))
 		((arrayp x)(listarray x))
 		((atom x) (list x))
@@ -277,22 +277,20 @@
     (or not-dim1 (setf (gethash 'dim1 table) t))
     table))
 
-;;range of atan should be [0,2*pi]
+;;; Range of atan should be [0,2*pi]
 (defun atan (y x)
   (let ((tem (cl:atan y x)))
     (if (>= tem 0)
 	tem
 	(+ tem (* 2 pi)))))
 
-;;range of atan2 should be (-pi,pi]
-;;CL manual says that's what lisp::atan is supposed to have.
+;;; Range of atan2 should be (-pi,pi]
+;;; CL manual says that's what lisp::atan is supposed to have.
 (deff atan2 #'cl:atan)
 
-;;exp is shadowed to save trouble for other packages--its declared special
+;;; exp is shadowed to save trouble for other packages--its declared special
 (deff exp #'cl:exp)
 
-(setq *read-default-float-format* 'double-float)
-
 #+clisp
 (progn
   ;; This used to be enabled, but
@@ -314,7 +312,121 @@
   ;; in those few cases when the mathematical result is exact although
   ;; one of the arguments is a floating-point number, such as (* 0
   ;; 1.618), (/ 0 1.618), (atan 0 1.0), (expt 2.0 0)
-  (setq custom:*floating-point-rational-contagion-ansi* t))
+  (setq custom:*floating-point-rational-contagion-ansi* t)
 
-(defmacro float (x &optional (y 1d0))
+  ;; When building maxima using with 'flonum being a 'long-float it may be
+  ;; useful to adjust the number of bits of precision that CLISP uses for
+  ;; long-floats.
+  #+nil
+  (setf (ext:long-float-digits) 128)
+  )
+
+;;;; Setup the mapping from the Maxima 'flonum float type to a CL float type.
+;;;;
+;;;; Add :flonum-log to *features* if you want flonum to be a
+;;;; long-float.  Or add :flonum-double-double if you want flonum to
+;;;; be a double-double (currently only for CMUCL).  Otherwise, you
+;;;; get double-float as the flonum type.
+;;;;
+;;;; Default double-float flonum.
+(eval-when (compile load eval)
+(setq *read-default-float-format* 'double-float)
+) ; eval-when
+
+#-(or flonum-long flonum-double-double)
+(progn
+;; Tell Lisp the float type for a 'flonum.
+(deftype flonum (&optional low high)
+  (cond (high
+	 `(double-float ,low ,high))
+	(low
+	 `(double-float ,low))
+	(t
+	 'double-float)))
+
+(defconstant most-positive-flonum most-positive-double-float)
+(defconstant most-negative-flonum most-negative-double-float)
+(defconstant least-positive-flonum least-positive-double-float)
+(defconstant least-negative-flonum least-negative-double-float)
+(defconstant flonum-epsilon double-float-epsilon)
+(defconstant least-positive-normalized-flonum least-positive-normalized-double-float)
+
+(defconstant flonum-exponent-marker #\D)
+)
+
+#+flonum-long
+(progn
+;;;; The Maxima 'flonum can be a CL 'long-float on the Scieneer CL or CLISP,
+;;;; but should be the same a 'double-float on other CL implementations.
+
+(eval-when (compile load eval)
+(setq *read-default-float-format* 'long-float)
+) ; eval-when
+
+;; Tell Lisp the float type for a 'flonum.
+(deftype flonum (&optional low high)
+  (cond (high
+	 `(long-float ,low ,high))
+	(low
+	 `(long-float ,low))
+	(t
+	 'long-float)))
+
+(defconstant most-positive-flonum most-positive-long-float)
+(defconstant most-negative-flonum most-negative-long-float)
+(defconstant least-positive-flonum least-positive-long-float)
+(defconstant least-negative-flonum least-negative-long-float)
+(defconstant flonum-epsilon long-float-epsilon)
+(defconstant least-positive-normalized-flonum least-positive-normalized-long-float)
+
+(defconstant flonum-exponent-marker #\L)
+
+)
+
+#+flonum-double-double
+(progn
+
+;;;; The Maxima 'flonum can be a 'kernel:double-double-float on the CMU CL.
+
+(eval-when (compile load eval)
+(setq *read-default-float-format* 'kernel:double-double-float)
+) ; eval-when
+
+;; Tell Lisp the float type for a 'flonum.
+(deftype flonum (&optional low high)
+  (cond (high
+	 `(kernel:double-double-float ,low ,high))
+	(low
+	 `(kernel:double-double-float ,low))
+	(t
+	 'kernel:double-double-float)))
+
+;; While double-double can represent number as up to
+;; most-positive-double-float, it can't really do operations on them
+;; due to the way multiplication and division are implemented.  (I
+;; don't think there's any workaround for that.)
+;;
+;; So, the largest number that can be used is the float just less than
+;; 2^1024/(1+2^27).  This is the number given here.
+(defconstant most-positive-double-double-hi
+  (scale-float (cl:float (1- 9007199187632128) 1d0) 944))
+
+(defconstant most-positive-flonum (cl:float most-positive-double-double-hi 1w0))
+(defconstant most-negative-flonum (cl:float (- most-positive-double-double-hi 1w0)))
+(defconstant least-positive-flonum (cl:float least-positive-double-float 1w0))
+(defconstant least-negative-flonum (cl:float least-negative-double-float 1w0))
+;; This is an approximation to a double-double epsilon.  Due to the
+;; way double-doubles are represented, epsilon is actually zero
+;; because 1+x = 1 only when x is zero.  But double-doubles only have
+;; 106 bits of precision, so we use that as epsilon.
+(defconstant flonum-epsilon (scale-float 1w0 -106))
+(defconstant least-positive-normalized-flonum (cl:float least-positive-normalized-double-float 1w0))
+
+(defconstant flonum-exponent-marker #\W)
+
+)
+
+;;;;
+(defmacro float (x &optional (y 1e0))
   `(cl:float ,x ,y))
+
