@@ -21,32 +21,48 @@
 
 ;; op and opr properties
 
-(mapc #'(lambda (x) (putprop (car x) (cadr x) 'op) (putprop (cadr x) (car x) 'opr))
-      '((mplus &+) (mminus &-) (mtimes &*) (mexpt &**) (mexpt &^)
-	(mnctimes |&.|) (rat &//) (mquotient &//) (mncexpt &^^)
-	(mequal &=) (mgreaterp &>) (mlessp &<) (mleqp &<=) (mgeqp &>=)
-	(mnotequal |&#|) (mand &and) (mor &or) (mnot &not) (msetq |&:|)
-	(mdefine |&:=|) (mdefmacro |&::=|) (mquote |&'|) (mlist &[)
-	(mset |&::|) (mfactorial &!) (marrow &->) (mprogn |&(|)
-	(mcond &if)))
+(defvar *opr-table* (make-hash-table :test #'equal))
+
+(defmfun getopr0 (x)
+  (or
+    (and (symbolp x) (get x 'opr))
+    (and (stringp x) (gethash x *opr-table*))))
+
+(defmfun getopr (x)
+  (or (getopr0 x) x))
+
+(defmfun putopr (x y)
+  (or
+    (and (symbolp x) (setf (get x 'opr) y))
+    (and (stringp x) (setf (gethash x *opr-table*) y))))
+
+(defmfun remopr (x)
+  (or
+    (and (symbolp x) (remprop x 'opr))
+    (and (stringp x) (remhash x *opr-table*))))
+
+(mapc #'(lambda (x) (putprop (car x) (cadr x) 'op) (putopr (cadr x) (car x)))
+      '((mplus "+") (mminus "-") (mtimes "*") (mexpt "**") (mexpt "^")
+	(mnctimes ".") (rat "/") (mquotient "/") (mncexpt "^^")
+	(mequal "=") (mgreaterp ">") (mlessp "<") (mleqp "<=") (mgeqp ">=")
+	(mnotequal "#") (mand "and") (mor "or") (mnot "not") (msetq ":")
+	(mdefine ":=") (mdefmacro "::=") (mquote "'") (mlist "[")
+	(mset "::") (mfactorial "!") (marrow "-->") (mprogn "(")
+	(mcond "if")))
 
 (mapc #'(lambda (x) (putprop (car x) (cadr x) 'op))
       '((mqapply $subvar) (bigfloat $bfloat)))
 
-(mapc #'(lambda (x) (putprop (car x) (cadr x) 'opr))
-      '((|&and| mand) (|&or| mor) (|&not| mnot) (|&if| mcond)))
-
 
 (setq $exptsubst nil
       $partswitch nil
       $inflag nil
       $gradefs '((mlist simp))
       $dependencies '((mlist simp))
-      atvars '(&@1 &@2 &@3 &@4)
+      atvars '($@1 $@2 $@3 $@4)
       atp nil
       islinp nil
       lnorecurse nil
-      &** '&^
       $derivsubst nil
       timesp nil
       $opsubst t
@@ -523,7 +539,7 @@
        (latvrs (length atvars))
        (l))
       ((not (< latvrs largl)) (nconc atvars l))
-    (setq l (cons (implode (cons '& (cons '@ (mexploden largl)))) l))))
+    (setq l (cons (implode (cons '$ (cons '@ (mexploden largl)))) l))))
 
 (defmfun notloreq (x)
   (or (atom x)
@@ -829,9 +845,6 @@
 (defmfun getop (x)
   (or (and (symbolp x) (get x 'op)) x))
 
-(defmfun getopr (x)
-  (or (and (symbolp x) (get x 'opr)) x))
-
 (defmfun $listp (x)
   (and (not (atom x))
        (not (atom (car x)))
@@ -953,11 +966,13 @@
   (atomchk (setq l (specrepcheck l)) '$delete t)
   (setq x (specrepcheck x)
 	l (cons (delsimp (car l)) (copy-list (cdr l))))
-  (do ((l1 (if (eq (caar l) 'mqapply) (cdr l) l) (cdr l1)))
+  (do ((l1 (if (eq (caar l) 'mqapply) (cdr l) l)))
       ((or (null (cdr l1)) (zerop n)) l)
-     (when (alike1 x (specrepcheck (cadr l1)))
-       (decf n)
-       (rplacd l1 (cddr l1)))))
+    (if (alike1 x (specrepcheck (cadr l1)))
+	(progn
+	  (decf n)
+	  (rplacd l1 (cddr l1)))
+	(setq l1 (cdr l1)))))
 
 (defmfun $length (e)
   (setq e (cond (($listp e) e)
@@ -1011,7 +1026,11 @@
 	((eq (caar e) 'rat) (fpcofrat e))
 	((eq (caar e) 'bigfloat) (fp2flo e))
 	((member (caar e) '(mexpt mncexpt) :test #'eq)
-	 (list (ncons (caar e)) ($float (cadr e)) (caddr e)))
+	 ;; avoid x^2 -> x^2.0, allow %e^%pi -> 23.14
+	 (let ((res (recur-apply #'$float e)))
+	   (if (floatp res)
+	       res
+	     (list (ncons (caar e)) ($float (cadr e)) (caddr e)))))
 	(t (recur-apply #'$float e))))
 
 (defmfun $coeff (e x &optional (n 1))
