@@ -84,11 +84,11 @@
 		      (return (simplify (list '(mplus)
 					      (simplify (list '(mtimes) '$%i ci))
 					      cr))))
-		    0d0 0d0))
+		    0.0 0.0))
 		  (t (return (list '(mlist simp)))))))
      (setq degree (cadr expr) nn (1+ degree))
-     (setq *pr-sl* (make-array nn :initial-element 0d0))
-     (setq *pi-sl* (make-array nn :initial-element 0d0))
+     (setq *pr-sl* (make-array nn :initial-element 0.0))
+     (setq *pi-sl* (make-array nn :initial-element 0.0))
      (or (catch 'notpoly
 	   (errset (do ((expr (cdr expr) (cddr expr)) (l) (%i (cadr res)))
 		       ((null expr))
@@ -104,15 +104,15 @@
 			    (setq complex t))))))
 	 ;;this should catch expressions like sin(x)-x
 	 (cpoly-err expr1))
-     (setq *shr-sl* (make-array nn :initial-element 0d0))
-     (setq *shi-sl* (make-array nn :initial-element 0d0))
-     (setq *qpr-sl* (make-array nn :initial-element 0d0))
-     (setq *hr-sl*  (make-array degree :initial-element 0d0))
-     (setq *qhr-sl* (make-array degree :initial-element 0d0))
+     (setq *shr-sl* (make-array nn :initial-element 0.0))
+     (setq *shi-sl* (make-array nn :initial-element 0.0))
+     (setq *qpr-sl* (make-array nn :initial-element 0.0))
+     (setq *hr-sl*  (make-array degree :initial-element 0.0))
+     (setq *qhr-sl* (make-array degree :initial-element 0.0))
      (when complex
-       (setq *qpi-sl* (make-array nn :initial-element 0d0))
-       (setq *hi-sl*  (make-array degree :initial-element 0d0))
-       (setq *qhi-sl* (make-array degree :initial-element 0d0)))
+       (setq *qpi-sl* (make-array nn :initial-element 0.0))
+       (setq *hi-sl*  (make-array degree :initial-element 0.0))
+       (setq *qhi-sl* (make-array degree :initial-element 0.0)))
      (setq nn degree)
      (if complex
 	 (setq res (errset (cpoly-sl degree)))
@@ -124,7 +124,7 @@
      (setq res nil)
      (cond ((not (zerop nn))
 	    (mtell "~%Only ~S out of ~S roots found " (- degree nn) degree)
-	    (setq expr 0d0)
+	    (setq expr 0.0)
 	    (do ((i 0 (1+ i)))
 		((> i nn))
 	      (setq expr
@@ -142,7 +142,7 @@
 				     (float (car den))
 				     (float (cadr den)))
 			  (simplify (list '(mplus) (simplify (list '(mtimes) '$%i ci)) cr)))
-			0d0 0d0)
+			0.0 0.0)
 		  res (cons expr res))))
      (do ((i degree (1- i)))
 	 ((= i nn))
@@ -172,7 +172,7 @@
 (defun cpoly-sl (degree) 
   ((lambda (logbas infin smalno are mre xx yy cosr sinr cr ci sr si tr
 	    ti zr zi bnd n polysc polysc1 conv) 
-     (setq mre (* 2d0 (sqrt 2d0) are)
+     (setq mre (* 2.0 (sqrt 2.0) are)
 	   yy (- xx))
      (do ((i degree (1- i)))
 	 ((not (and (zerop (aref *pr-sl* i)) (zerop (aref *pi-sl* i))))
@@ -224,12 +224,19 @@
        (setf (aref *pr-sl* i) (scale-float (aref *pr-sl* i) j))
        (setf (aref *pi-sl* i) (scale-float (aref *pi-sl* i) j)))
      nn)
-   (log 2d0)
-   most-positive-double-float
-   least-positive-double-float
-   double-float-epsilon
-   0d0 0.70710677d0 0d0 -0.069756474d0 0.99756405d0
-   0d0 0d0 0d0 0d0 0d0 0d0 0d0 0d0 0d0 0 0 0 nil))
+   (log 2.0)
+   most-positive-flonum
+   least-positive-flonum
+   flonum-epsilon
+   0.0
+   ;; sqrt(0.5)
+   0.7071067811865475244008443621048490392848359376884740365883398689953662392310535194251937671638207863675069231154561485124624180279253686063220607485499679157066113329637527963778999752505763910302857350547799858029851372672984310073642587093204445993047761646152421543571607254198813018139976257039948436267
+   0.0
+   ;; cos(94deg)
+   -0.0697564737441253007759588351941433286009032016527965250436172961370711270667891229125378568280742923028942076107741717160209821557740512756197740925891665208235244345674420755726285778495732000059330205461129612198466216775458241726113210999152981126990497403794217445425671287263223529689424188857433131142804
+   ;; sin(94deg)
+   0.9975640502598242476131626806442550263693776603842215362259956088982191814110818430852892116754760376426967121558233963175758546629687044962793968705262246733087781690124900795021134880736278349857522534853084644420433826380899280074903993378273609249428279246573946968632240992430211366514177713203298481315
+   0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0 0 0 nil))
 
 (defun noshft-sl (l1) 
   (do ((i 0 (1+ i))
@@ -241,7 +248,7 @@
   (do ((jj 1 (1+ jj)))
       ((> jj l1))
     (cond ((> (cmod-sl (aref *hr-sl* n) (aref *hi-sl* n))
-	      (* 10d0 are (cmod-sl (aref *pr-sl* n) (aref *pi-sl* n))))
+	      (* 10.0 are (cmod-sl (aref *pr-sl* n) (aref *pi-sl* n))))
 	   (cdivid-sl (- (aref *pr-sl* nn)) (- (aref *pi-sl* nn)) (aref *hr-sl* n) (aref *hi-sl* n))
 	   (setq tr cr ti ci)
 	   (do ((j n (1- j)) (t1) (t2))
@@ -255,8 +262,8 @@
 		 ((> 1 j))
 	       (setf (aref *hr-sl* j) (aref *hr-sl* (1- j)))
 	       (setf (aref *hi-sl* j) (aref *hi-sl* (1- j))))
-	     (setf (aref *hr-sl* 0) 0d0)
-	     (setf (aref *hi-sl* 0) 0d0))))) 
+	     (setf (aref *hr-sl* 0) 0.0)
+	     (setf (aref *hi-sl* 0) 0.0))))) 
 
 (defun fxshft-sl (l2) 
   ((lambda (test pasd otr oti svsr svsi bool pvr pvi) 
@@ -291,7 +298,7 @@
 		    ((setq pasd nil))))))
      (or conv (vrshft-sl 10))
      nil)
-   t nil 0d0 0d0 0d0 0d0 nil 0d0 0d0)) 
+   t nil 0.0 0.0 0.0 0.0 nil 0.0 0.0)) 
 
 (defun vrshft-sl (l3) 
   (setq conv nil sr zr si zi)
@@ -301,12 +308,12 @@
       ((> i l3))
     (polyev-sl)
     (setq mp (cmod-sl pvr pvi) ms (cmod-sl sr si))
-    (cond ((> (* 20d0 (errev-sl ms mp)) mp)
+    (cond ((> (* 20.0 (errev-sl ms mp)) mp)
 	   (setq conv t zr sr zi si)
 	   (return t)))
     (cond ((= i 1) (setq omp mp))
 	  ((or bool1 (> omp mp) (not (< relstp 0.05)))
-	   (if (> (* 0.1d0 mp) omp)
+	   (if (> (* 0.1 mp) omp)
 	       (return t)
 	       (setq omp mp)))
 	  (t (setq tp relstp bool1 t)
@@ -332,9 +339,9 @@
        (hvr (setf (aref *qhr-sl* 0) (aref *hr-sl* 0)))
        (hvi (setf (aref *qhi-sl* 0) (aref *hi-sl* 0))))
       ((> i n)
-       (setq bool (not (> (cmod-sl hvr hvi) (* 10d0 are (cmod-sl (aref *hr-sl* n) (aref *hi-sl* n))))))
+       (setq bool (not (> (cmod-sl hvr hvi) (* 10.0 are (cmod-sl (aref *hr-sl* n) (aref *hi-sl* n))))))
        (cond ((not bool) (cdivid-sl (- pvr) (- pvi) hvr hvi) (setq tr cr ti ci))
-	     (t (setq tr 0d0 ti 0d0)))
+	     (t (setq tr 0.0 ti 0.0)))
        nil)
     (setq $t (- (+ (aref *hr-sl* i) (* hvr sr)) (* hvi si)))
     (setf (aref *qhi-sl* i) (setq hvi (+ (aref *hi-sl* i) (* hvr si) (* hvi sr))))
@@ -345,8 +352,8 @@
 		  ((> j n))
 		(setf (aref *hr-sl* j) (aref *qhr-sl* (1- j)))
 		(setf (aref *hi-sl* j) (aref *qhi-sl* (1- j))))
-	      (setf (aref *hr-sl* 0) 0d0)
-	      (setf (aref *hi-sl* 0) 0d0))
+	      (setf (aref *hr-sl* 0) 0.0)
+	      (setf (aref *hi-sl* 0) 0.0))
 	(t (do ((j 1. (1+ j)) (t1) (t2))
 	       ((> j n))
 	     (setq t1 (aref *qhr-sl* (1- j)) t2 (aref *qhi-sl* (1- j)))
@@ -381,28 +388,28 @@
 	    (cond ((> x xm) (setq x xm)))))
      (do ((f))
 	 (nil)
-       (setq xm (* 0.1d0 x) f (aref *shr-sl* 0))
+       (setq xm (* 0.1 x) f (aref *shr-sl* 0))
        (do ((i 1 (1+ i))) ((> i nn)) (setq f (+ (aref *shr-sl* i) (* f xm))))
-       (cond ((not (< 0d0 f)) (return t)))
+       (cond ((not (< 0.0 f)) (return t)))
        (setq x xm))
      (do ((dx x) (df) (f))
-	 ((> 5d-3 (abs (/ dx x))) x)
+	 ((> 5e-3 (abs (/ dx x))) x)
        (setq f (aref *shr-sl* 0) df f)
        (do ((i 1 (1+ i)))
 	   ((> i n))
 	 (setq f (+ (* f x) (aref *shr-sl* i)) df (+ (* df x) f)))
        (setq f (+ (* f x) (aref *shr-sl* nn)) dx (/ f df) x (- x dx))))
    (exp (/ (- (log (aref *shr-sl* nn)) (log (aref *shr-sl* 0))) (float nn)))
-   0d0)) 
+   0.0)) 
 
 (defun scale-sl ()
-  (do ((i 0 (1+ i)) (j 0) (x 0d0) (dx 0d0))
+  (do ((i 0 (1+ i)) (j 0) (x 0.0) (dx 0.0))
       ((> i nn)
        (setq x (/ x (float (- (1+ nn) j)))
 	     dx (/ (- (log (aref *shr-sl* nn)) (log (aref *shr-sl* 0))) (float nn))
-	     polysc1 (floor (+ 0.5d0 (/ dx logbas)))
-	     x (+ x (* (float (* polysc1 nn)) logbas 0.5d0))
-	     polysc (floor (+ 0.5d0 (/ x logbas)))))
+	     polysc1 (floor (+ 0.5 (/ dx logbas)))
+	     x (+ x (* (float (* polysc1 nn)) logbas 0.5))
+	     polysc (floor (+ 0.5 (/ x logbas)))))
     (cond ((zerop (aref *shr-sl* i)) (setq j (1+ j)))
 	  (t (setq x (+ x (log (aref *shr-sl* i)))))))
   (do ((i nn (1- i)) (j (- polysc) (+ j polysc1)))
@@ -426,7 +433,7 @@
 		  cr (/ br bi) 
 		  br (- ai (* ar r1)) 
 		  ci (/ br bi)))))
-   0d0)
+   0.0)
   nil) 
 
 (defun cmod-sl (ar ai) 
@@ -434,15 +441,16 @@
 	ai (abs ai))
   (cond ((> ai ar) (setq ar (/ ar ai)) (* ai (sqrt (1+ (* ar ar)))))
 	((> ar ai) (setq ai (/ ai ar)) (* ar (sqrt (1+ (* ai ai)))))
-	((* 1.41421357d0 ar)))) 
+	((* 1.4142135623730950488016887242096980785696718753769480731766797379907324784621070388503875343276415727350138462309122970249248360558507372126441214970999358314132226659275055927557999505011527820605714701095599716059702745345968620147285174186408891986095523292304843087143214508397626036279952514079896872534 ; sqrt(2.0)
+	  ar))))
 
-;;;this is the algorithm for doing real polynomials.
-;;;it is algorithm 493 from acm toms vol 1 p 178 (1975) by jenkins.
-;;;note that array indexing starts from 0.
-;;;the names of the arrays have been changed to be the same as for cpoly.
-;;;the correspondence is:  p - pr-sl, qp - qpr-sl, k - hr-sl, qk - qhr-sl,
-;;;svk - shr-sl, temp - shi-sl.  the roots are put in pr-sl and pi-sl.
-;;;the variable si appears not to be used here
+;;; This is the algorithm for doing real polynomials.
+;;; It is algorithm 493 from acm toms vol 1 p 178 (1975) by jenkins.
+;;; Note that array indexing starts from 0.
+;;; The names of the arrays have been changed to be the same as for cpoly.
+;;; The correspondence is:  p - pr-sl, qp - qpr-sl, k - hr-sl, qk - qhr-sl,
+;;; svk - shr-sl, temp - shi-sl.  the roots are put in pr-sl and pi-sl.
+;;; The variable si appears not to be used here
 
 (declare-top (special sr u v a b c d a1 a3 a7 e f g h szr szi lzr lzi are
 		      mre n nn nz ui vi s $polyfactor)) 
@@ -462,7 +470,7 @@
 		 (cond ((and $polyfactor (not (zerop szi)))
 			(setf (aref *pr-sl* 2) (/ (aref *pr-sl* 2) (aref *pr-sl* 0)))
 			(setf (aref *pr-sl* 1) (/ (aref *pr-sl* 1) (aref *pr-sl* 0)))
-			(setf (aref *pi-sl* 2) 1d0))
+			(setf (aref *pi-sl* 2) 1.0))
 		       (t (setf (aref *pr-sl* 2) szr)
 			  (setf (aref *pi-sl* 2) szi)
 			  (setf (aref *pr-sl* 1) lzr)
@@ -482,7 +490,7 @@
 	 (cond (zerok (do ((j n (1- j)))
 			  ((< j 1))
 			(setf (aref *hr-sl* j) (aref *hr-sl* (1- j))))
-		      (setf (aref *hr-sl* 0) 0d0)
+		      (setf (aref *hr-sl* 0) 0.0)
 		      (setq zerok (zerop (aref *hr-sl* n))))
 	       (t (setq t1 (- (/ aa cc)))
 		  (do ((j n (1- j)))
@@ -490,7 +498,7 @@
 		    (setf (aref *hr-sl* j) (+ (* t1 (aref *hr-sl* (1- j))) (aref *pr-sl* j))))
 		  (setf (aref *hr-sl* 0) (aref *pr-sl* 0))
 		  (setq zerok (not (> (abs (aref *hr-sl* n))
-				      (* (abs bb) are 10d0)))))))
+				      (* (abs bb) are 10.0)))))))
        (do ((i 0 (1+ i))) ((> i n)) (setf (aref *shi-sl* i) (aref *hr-sl* i)))
        (do ((cnt 1 (1+ cnt)))
 	   ((> cnt 20.) (setq conv1 nil))
@@ -498,7 +506,7 @@
 		      (- (* cosr xx) (* sinr yy))
 		    (setq yy (+ (* sinr xx) (* cosr yy)))) 
 	       sr (* bnd xx) 
-	       u (* -2d0 sr) 
+	       u (* -2.0 sr) 
 	       v bnd)
 	 (fxshfr-sl (* 20 cnt))
 	 (cond ((> nz 0)
@@ -510,7 +518,7 @@
 		       (cond ((and $polyfactor (not (zerop szi)))
 			      (setf (aref *pr-sl* nn) v)
 			      (setf (aref *pr-sl* n) u)
-			      (setf (aref *pi-sl* nn) 1d0)))))
+			      (setf (aref *pi-sl* nn) 1.0)))))
 		(setq nn (- nn nz) n (1- nn))
 		(do ((i 0 (1+ i))) ((> i nn)) (setf (aref *pr-sl* i) (aref *qpr-sl* i)))
 		(return nil)))
@@ -531,12 +539,19 @@
      (do ((i 0 (1+ i)) (j (- polysc (* polysc1 degree)) (+ j polysc1)))
 	 ((> i nn))
        (setf (aref *pr-sl* i) (scale-float (aref *pr-sl* i) j))))
-   (log 2d0)
-   most-positive-double-float
-   least-positive-double-float
-   double-float-epsilon
-   0d0 0.70710677d0 0d0 -0.069756474d0 0.99756405d0
-   0d0 0d0 0d0 0d0 0d0 0d0 0d0 0d0 0d0 0d0 0d0 0d0 0 0 0 0 0 t))  
+   (log 2.0)
+   most-positive-flonum
+   least-positive-flonum
+   flonum-epsilon
+   0.0
+   ;; sqrt(0.5)
+   0.7071067811865475244008443621048490392848359376884740365883398689953662392310535194251937671638207863675069231154561485124624180279253686063220607485499679157066113329637527963778999752505763910302857350547799858029851372672984310073642587093204445993047761646152421543571607254198813018139976257039948436267
+   0.0
+   ;; cos(94deg)
+   -0.0697564737441253007759588351941433286009032016527965250436172961370711270667891229125378568280742923028942076107741717160209821557740512756197740925891665208235244345674420755726285778495732000059330205461129612198466216775458241726113210999152981126990497403794217445425671287263223529689424188857433131142804
+   ;; sin(94deg)
+   0.9975640502598242476131626806442550263693776603842215362259956088982191814110818430852892116754760376426967121558233963175758546629687044962793968705262246733087781690124900795021134880736278349857522534853084644420433826380899280074903993378273609249428279246573946968632240992430211366514177713203298481315
+   0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0 0 0 0 0 t))  
 
 (defun fxshfr-sl (l2) 
   ((lambda (my-type a b c d e f g h a1 a3 a7)
@@ -545,8 +560,8 @@
      (quadsd-sl)
      (calcsc-sl)
      (do ((j 1 (1+ j))
-	  (betav 0.25d0)
-	  (betas 0.25d0)
+	  (betav 0.25)
+	  (betas 0.25)
 	  (oss sr)
 	  (ovv v)
 	  (tvv) (tss) (ss) (vv) (tv) (ts) (ots) (otv)
@@ -556,16 +571,16 @@
        (calcsc-sl)
        (newest-sl)
        (setq vv vi
-	     ss 0d0)
+	     ss 0.0)
        (or (zerop (aref *hr-sl* n))
 	   (setq ss (- (/ (aref *pr-sl* nn) (aref *hr-sl* n)))))
-       (setq tv 1d0 ts 1d0)
+       (setq tv 1.0 ts 1.0)
        (cond ((not (or (= j 1) (= my-type 3)))
 	      (or (zerop vv) (setq tv (abs (/ (- vv ovv) vv))))
 	      (or (zerop ss) (setq ts (abs (/ (- ss oss) ss))))
-	      (setq tvv 1d0)
+	      (setq tvv 1.0)
 	      (and (< tv otv) (setq tvv (* tv otv)))
-	      (setq tss 1d0)
+	      (setq tss 1.0)
 	      (and (< ts ots) (setq tss (* ts ots)))
 	      (setq vpass (< tvv betav) spass (< tss betas))
 	      (cond ((or spass vpass)
@@ -580,7 +595,7 @@
 			    (cond (bool (quadit-sl)
 					(and (> nz 0) (return t))
 					(setq vtry t
-					      betav (* 0.25d0 betav))
+					      betav (* 0.25 betav))
 					(cond ((or stry (not spass))
 					       (setq l50 t))
 					      (t (do ((i 0 (1+ i)))
@@ -590,7 +605,7 @@
 			    (cond ((not l50)
 				   (setq iflag (realit-sl))
 				   (and (> nz 0) (return t))
-				   (setq stry t betas (* 0.25d0 betas))
+				   (setq stry t betas (* 0.25 betas))
 				   (cond ((zerop iflag) (setq l50 t))
 					 (t (setq ui (- (+ s s)) 
 						  vi (* s s))))))
@@ -608,32 +623,32 @@
 	     oss ss
 	     otv tv
 	     ots ts)))
-   0 0d0 0d0 0d0 0d0 0d0 0d0 0d0 0d0 0d0 0d0 0d0))
+   0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0))
 
 (defun quadit-sl nil 
   (setq nz 0 u ui v vi)
   (do ((tried) (j 0) (ee) (zm) (t1) (mp) (relstp) (omp))
       (nil)
-    (quad-sl 1d0 u v)
-    (and (> (abs (- (abs szr) (abs lzr))) (* 1d-2 (abs lzr)))
+    (quad-sl 1.0 u v)
+    (and (> (abs (- (abs szr) (abs lzr))) (* 1e-2 (abs lzr)))
 	 (return nil))
     (quadsd-sl)
     (setq mp (+ (abs (- a (* szr b))) (abs (* szi b))) 
 	  zm (sqrt (abs v)) 
-	  ee (* 2d0 (abs (aref *qpr-sl* 0)))
+	  ee (* 2.0 (abs (aref *qpr-sl* 0)))
 	  t1 (- (* szr b)))
     (do ((i 1 (1+ n)))
 	((> i n)) (setq ee (+ (* ee zm) (abs (aref *qpr-sl* i)))))
     (setq ee (+ (* ee zm) (abs (+ a t1))) 
-	  ee (- (* (+ (* 5d0 mre) (* 4d0 are)) ee)
-		 (* (+ (* 5d0 mre) (* 2d0 are))
+	  ee (- (* (+ (* 5.0 mre) (* 4.0 are)) ee)
+		 (* (+ (* 5.0 mre) (* 2.0 are))
 		     (+ (abs (+ a t1)) (* (abs b) zm)))
-		 (* -2d0 are (abs t1))))
-    (cond ((not (> mp (* 2d1 ee))) (setq nz 2)
+		 (* -2.0 are (abs t1))))
+    (cond ((not (> mp (* 2e1 ee))) (setq nz 2)
 	   (return nil)))
     (setq j (1+ j))
     (and (> j 20) (return nil))
-    (cond ((not (or (< j 2) (> relstp 1d-2) (< mp omp) tried))
+    (cond ((not (or (< j 2) (> relstp 1e-2) (< mp omp) tried))
 	   (and (< relstp are) (setq relstp are))
 	   (setq relstp (sqrt relstp) 
 		 u (- u (* u relstp)) 
@@ -665,13 +680,13 @@
 	  ee (* (/ mre (+ are mre)) (abs (aref *qpr-sl* 0))))
     (do ((i 1 (1+ i)))
 	((> i nn)) (setq ee (+ (* ee ms) (abs (aref *qpr-sl* i)))))
-    (cond ((not (> mp (* 2d1 (- (* (+ are mre) ee) (* mre mp)))))
-	   (setq nz 1 szr s szi 0d0)
+    (cond ((not (> mp (* 2e1 (- (* (+ are mre) ee) (* mre mp)))))
+	   (setq nz 1 szr s szi 0.0)
 	   (return 0)))
     (setq j (1+ j))
     (and (> j 10) (return 0))
     (cond ((not (or (< j 2)
-		    (> (abs t1) (* 1d-3 (abs (- s t1))))
+		    (> (abs t1) (* 1e-3 (abs (- s t1))))
 		    (not (> mp omp))))
 	   (return 1)))
     (setq omp mp kv (aref *hr-sl* 0))
@@ -680,21 +695,21 @@
 	((> i n))
       (setq kv (+ (* kv s) (aref *hr-sl* i)))
       (setf (aref *qhr-sl* i) kv))
-    (cond ((> (abs kv) (* (abs (aref *hr-sl* n)) 1d1 are))
+    (cond ((> (abs kv) (* (abs (aref *hr-sl* n)) 1e1 are))
 	   (setq t1 (- (/ pv kv)))
 	   (setf (aref *hr-sl* 0) (aref *qpr-sl* 0))
 	   (do ((i 1 (1+ i)))
 	       ((> i n))
 	     (setf (aref *hr-sl* i)
 		    (+ (* t1 (aref *qhr-sl* (1- i))) (aref *qpr-sl* i)))))
-	  (t (setf (aref *hr-sl* 0) 0d0)
+	  (t (setf (aref *hr-sl* 0) 0.0)
 	     (do ((i 1 (1+ i)))
 		 ((> i n)) (setf (aref *hr-sl* i) (aref *qhr-sl* (1- i))))))
     (setq kv (aref *hr-sl* 0))
     (do ((i 1 (1+ i)))
 	((> i n)) (setq kv (+ (* kv s) (aref *hr-sl* i))))
-    (setq t1 0d0)
-    (and (> (abs kv) (* (abs (aref *hr-sl* n)) 10d0 are))
+    (setq t1 0.0)
+    (and (> (abs kv) (* (abs (aref *hr-sl* n)) 10.0 are))
 	 (setq t1 (- (/ pv kv))))
     (setq s (+ s t1))))
 
@@ -710,8 +725,8 @@
     (setq c0 (- (aref *hr-sl* i) (* u c) (* v d)))
     (setf (aref *qhr-sl* i) c0)
     (setq d c c c0))
-  (cond ((not (or (> (abs c) (* (abs (aref *hr-sl* n)) 1d2 are))
-		  (> (abs d) (* (abs (aref *hr-sl* (1- n))) 1d2 are))))
+  (cond ((not (or (> (abs c) (* (abs (aref *hr-sl* n)) 1e2 are))
+		  (> (abs d) (* (abs (aref *hr-sl* (1- n))) 1e2 are))))
 	 (setq my-type 3))
 	((not (< (abs d) (abs c)))
 	 (setq my-type 2 
@@ -735,11 +750,11 @@
 (defun nextk-sl ()
   (declare (special my-type))
   (cond ((= my-type 3)
-	 (setf (aref *hr-sl* 0) 0d0)
-	 (setf (aref *hr-sl* 1) 0d0)
+	 (setf (aref *hr-sl* 0) 0.0)
+	 (setf (aref *hr-sl* 1) 0.0)
 	 (do ((i 2 (1+ i)))
 	     ((> i n)) (setf (aref *hr-sl* i) (aref *qhr-sl* (- i 2)))))
-	((> (abs a1) (* (abs (if (= my-type 1) b a)) 1d1 are))
+	((> (abs a1) (* (abs (if (= my-type 1) b a)) 1e1 are))
 	 (setq a7 (/ a7 a1) a3 (/ a3 a1))
 	 (setf (aref *hr-sl* 0) (aref *qpr-sl* 0))
 	 (setf (aref *hr-sl* 1) (- (aref *qpr-sl* 1) (* a7 (aref *qpr-sl* 0))))
@@ -749,7 +764,7 @@
 		  (+ (* a3 (aref *qhr-sl* (- i 2)))
 		      (- (* a7 (aref *qpr-sl* (1- i))))
 		      (aref *qpr-sl* i)))))
-	(t (setf (aref *hr-sl* 0) 0d0)
+	(t (setf (aref *hr-sl* 0) 0.0)
 	   (setf (aref *hr-sl* 1) (- (* a7 (aref *qpr-sl* 0))))
 	   (do ((i 2 (1+ i)))
 	       ((> i n))
@@ -761,7 +776,7 @@
 (defun newest-sl ()
   (declare (special my-type))
   ((lambda (a4 a5 b1 b2 c1 c2 c3 c4) 
-     (cond ((= my-type 3) (setq ui 0d0 vi 0d0))
+     (cond ((= my-type 3) (setq ui 0.0 vi 0.0))
 	   (t (if (= my-type 2)
 		  (setq a4 (+ (* (+ a g) f) h) 
 			a5 (+ (* (+ f u) c) (* v d)))
@@ -775,13 +790,13 @@
 		    c4 (- c1 c2 c3) 
 		    c1 (+ a5 (* b1 a4) (- c4)))
 	      (if (zerop c1)
-		  (setq ui 0d0 vi 0d0)
+		  (setq ui 0.0 vi 0.0)
 		  (setq ui (- u (/ (+ (* u (+ c3 c2))
 				      (* v (+ (* b1 a1) (* b2 a7))))
 				   c1))
 			vi (* v (1+ (/ c4 c1)))))))
      nil)
-   0d0 0d0 0d0 0d0 0d0 0d0 0d0 0d0)) 
+   0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0)) 
 
 (defun quadsd-sl ()
   (setq b (aref *pr-sl* 0))
@@ -797,28 +812,28 @@
 	  a c0)))
 
 (defun quad-sl (a0 b1 c0) 
-  (setq szr 0d0 szi 0d0 lzr 0d0 lzi 0d0)
-  ((lambda (b0 d0 e) 
+  (setq szr 0.0 szi 0.0 lzr 0.0 lzi 0.0)
+  ((lambda (b0 l0 e) 
      (cond ((zerop a0) (or (zerop b1) (setq szr (- (/ c0 b1)))))
 	   ((zerop c0) (setq lzr (- (/ b1 a0))))
-	   (t (setq b0 (/ b1 2d0))
+	   (t (setq b0 (/ b1 2.0))
 	      (cond ((< (abs b0) (abs c0))
 		     (setq e a0)
-		     (and (< c0 0.0d0) (setq e (- a0)))
+		     (and (< c0 0.0) (setq e (- a0)))
 		     (setq e (- (* b0 (/ b0 (abs c0))) e) 
-			   d0 (* (sqrt (abs e)) (sqrt (abs c0)))))
-		    (t (setq e (- 1d0 (* (/ a0 b0) (/ c0 b0))) 
-			     d0 (* (sqrt (abs e)) (abs b0)))))
-	      (cond ((< e 0.0d0)
+			   l0 (* (sqrt (abs e)) (sqrt (abs c0)))))
+		    (t (setq e (- 1.0 (* (/ a0 b0) (/ c0 b0))) 
+			     l0 (* (sqrt (abs e)) (abs b0)))))
+	      (cond ((< e 0.0)
 		     (setq szr (- (/ b0 a0)) 
 			   lzr szr 
-			   szi (abs (/ d0 a0)) 
+			   szi (abs (/ l0 a0)) 
 			   lzi (- szi)))
-		    (t (or (< b0 0d0) (setq d0 (- d0)))
-		       (setq lzr (/ (- d0 b0) a0))
+		    (t (or (< b0 0.0) (setq l0 (- l0)))
+		       (setq lzr (/ (- l0 b0) a0))
 		       (or (zerop lzr) (setq szr (/ c0 lzr a0)))))))
      nil)
-   0d0 0d0 0d0)) 
+   0.0 0.0 0.0)) 
 
 (declare-top (unspecial logbas infin smalno are mre cr ci sr si tr ti zr zi
 			n nn bool conv pvr pvi polysc polysc1 sr u v
