@@ -69,10 +69,10 @@
 ;; Parameters which control how boxes, absolute value signs,
 ;; evaluation-at-a-point bars, and matrices are drawn.
 
-(defmvar $boxchar '|&"|  "Character used for drawing boxes.")
-(defmvar $absboxchar '|&!| "Character used for drawing absolute value signs and 'evaluation at' signs.")
-(defmvar $lmxchar '|&[|  "Character used for drawing the left edge of a matrix.")
-(defmvar $rmxchar '|&]|  "Character used for drawing the right edge of a matrix.")
+(defmvar $boxchar "\""  "Character used for drawing boxes.")
+(defmvar $absboxchar "!" "Character used for drawing absolute value signs and 'evaluation at' signs.")
+(defmvar $lmxchar "["  "Character used for drawing the left edge of a matrix.")
+(defmvar $rmxchar "]"  "Character used for drawing the right edge of a matrix.")
 
 ;; These variables are bound within Macsyma Listeners since they are different
 ;; for each window.  Set them here, anyway, so that RETRIEVE can be called from
@@ -219,20 +219,22 @@
 (defmfun makestring (atom)
   (let (dummy)
     (cond ((numberp atom) (exploden atom))
+      ((stringp atom)
+       (setq dummy (coerce atom 'list))
+       (if $stringdisp
+         (cons #\" (nconc dummy (list #\")))
+         dummy))
 	  ((not (symbolp atom)) (exploden atom))
 	  ((and (setq dummy (get atom 'reversealias))
 		(not (and (member atom $aliases :test #'eq) (get atom 'noun))))
-	   (exploden dummy))
+	   (exploden (stripdollar dummy)))
 	  ((not (eq (getop atom) atom))
-	   (setq dummy (exploden (getop atom)))
-	   (if (char= #\& (car dummy))
-	       (cons #\" (nconc (cdr dummy) (list #\")))
-	       (cdr dummy)))
+       (makestring (getop atom)))
 	  (t (setq dummy (exploden atom))
-	     (cond ((char= #\$ (car dummy)) (cdr dummy))
-		   ((and $stringdisp (char= #\& (car dummy)))
-		    (cons #\" (nconc (cdr dummy) (list #\"))))
-		   ((or (char= #\% (car dummy)) (char= #\& (car dummy))) (cdr dummy))
+	     (cond
+           ((null dummy) nil)
+           ((char= #\$ (car dummy)) (cdr dummy))
+		   ((char= #\% (car dummy)) (cdr dummy))
 		   ($lispdisp (cons #\? dummy))
 		   (t dummy))))))
 
@@ -339,34 +341,104 @@
   (cond ((null (cddr form))
 	 (dimension-function form result))
 	(t
-	 (prog (dissym (symlength 0) (w 0) (h 0) (d 0))
+	 (prog (dissym (symlength 0) (w 0) (h 0) (d 0) helper)
 	    (setq dissym (safe-get (caar form) 'dissym)
 		  symlength (length dissym)
-		  result (dimnary (cadr form) result lop (caar form) (caar form) 0)
+
+          ;; Look for a helper function. Fall back on default if none found.
+          helper (or (safe-get (caar form) 'dimension-nary-helper) 'dimnary)
+
+		  result (funcall helper (cadr form) result lop (caar form) (caar form) 0)
 		  w width
 		  h height
-		  d depth)
+		  d depth
+          )
 	    (do ((l (cddr form) (cdr l)))
 		(nil)
 	      (checkbreak result w)
 	      (setq result (revappend dissym result) w (+ symlength w))
 	      (cond ((null (cdr l))
-		     (setq result (dimnary (car l) result (caar form) (caar form) rop w)
+		     (setq result (funcall helper (car l) result (caar form) (caar form) rop w)
 			   width (+ w width)
 			   height (max h height)
 			   depth (max d depth))
 		     (return t))
 		    (t
-		     (setq result (dimnary (car l) result (caar form) (caar form) (caar form) w)
+		     (setq result (funcall helper (car l) result (caar form) (caar form) (caar form) w)
 			   w (+ w width)
 			   h (max h height)
 			   d (max d depth)))))
 	    (return result)))))
 
-;; Check for (* A (* B C)) --> A*(B*C)
+;; Output for MTEXT, generated for example by print(...).
+;; Suppress parentheses which would be generated by DIMNARY.
+
+(defun dimnary-mtext (form result lop op rop w)
+  (declare (ignore op))
+  (dimension form result lop rop w right))
+
+;; Output for general n-ary operators.
+;; Heuristic: if some argument is displayed as an infix or n-ary operator,
+;; then put parentheses around that argument (even if parentheses are not
+;; necessary, according to operator precedence, to disambiguate the output).
+;;
+;; Examples:
+;;
+;; a * b * c;                   --> a b c
+;; a . b . c;                   --> a . b . c
+;; a * b . c;                   --> a (b . c)
+;; (a * b) . c;                 --> (a b) . c
+;; a . b * c;                   --> (a . b) c
+;; a . (b * c);                 --> a . (b c)
+;; a . b * c . d;               --> (a . b) (c . d)
+;; a * b . c * d;               --> a (b . c) d
+;; (a * b) . (c * d);           --> (a b) . (c d)
+;; infix ("@@");
+;; a @@ b * c @@ d;             --> (a @@ b) (c @@ d)
+;; a @@ b . c @@ d;             --> (a @@ b) . (c @@ d)
+;; a * b @@ c . d;              --> a ((b @@ c) . d)
+;; (a * b) @@ (c . d);          --> (a b) @@ (c . d)
 
 (defun dimnary (form result lop op rop w)
-  (if (and (not (atom form)) (eq (caar form) op))
+  (declare (ignore op))
+  (if (and (consp form)
+	   (member (safe-get (caar form) 'dimension) '(dimension-infix dimension-nary)))
+      (dimension-paren form result)
+      (dimension form result lop rop w right)))
+
+;; Output for Boolean n-ary operators.
+;; Heuristic: if some argument is displayed as an infix or n-ary operator,
+;; or the operator of the argument is MNOT, then put parentheses around that argument
+;; (even if parentheses are not necessary, according to operator precedence,
+;; to disambiguate the output).
+;;
+;; Examples:
+;;
+;; a and b and c;               --> a and b and c
+;; a or b or c;                 --> a or b or c
+;; a or b and c;                --> a or (b and c)
+;; (a or b) and c;              --> (a or b) and c
+;; a and b or c;                --> (a and b) or c
+;; a and (b or c);              --> a and (b or c)
+;; a and b or c and d;          --> (a and b) or (c and d)
+;; a or b and c or d;           --> a or (b and c) or d
+;; (a or b) and (c or d);       --> (a or b) and (c or d)
+;; a < 0 and b < 0 or c < 0;    --> ((a < 0) and (b < 0)) or (c < 0)
+;; a < 0 and not foo;           --> (a < 0) and (not foo)
+;; a < 0 or not foo;            --> (a < 0) or (not foo)
+;; infix ("=>");
+;; a => b and a => c;           --> (a => b) and (a => c)
+;; a => b or not b => c;        --> (a => b) or (not b => c)
+;; a and b => c and d;          --> a and (b => c) and d
+;; a or b => c and d;           --> a or ((b => c) and d)
+;; (a and b) => (c and d);      --> (a and b) => (c and d)
+;; if a and b then 1 else 0;    --> if a and b then 1 else 0
+
+(defun dimnary-boolean (form result lop op rop w)
+  (declare (ignore op))
+  (if (and (consp form)
+	   (or (member (safe-get (caar form) 'dimension) '(dimension-infix dimension-nary))
+	       (eq (caar form) 'mnot)))
       (dimension-paren form result)
       (dimension form result lop rop w right)))
 
@@ -537,7 +609,7 @@
 ;;            | ^              |
 ;;   <- X1 -> | | H1           |
 ;;            | | D1           |
-;;	      | v	       |
+;;            | v              |
 ;;            ------------------
 ;;    ----------------------------------
 ;; (Likewise for X2, H2, D2, W2 in the denominator)
@@ -681,7 +753,7 @@
      (unless (checkfit (+ 1 w width))
        (return (dimension-function form result)))
      (setq result (cons (cons 0 (cons (- 0 1 d) eqs))
-			(cons `(d-vbar ,(1+ h) ,(1+ d) ,(char (symbol-name $absboxchar) 1)) exp))
+			(cons `(d-vbar ,(1+ h) ,(1+ d) ,(car (coerce $absboxchar 'list))) exp))
 	   width (+ 1 w width)
 	   height (1+ h)
 	   depth (+ 1 d depth))
@@ -775,7 +847,9 @@
 (displa-def mlessp    dimension-infix  " < ")
 (displa-def mnot      dimension-prefix "not ")
 (displa-def mand      dimension-nary   " and ")
+(defprop mand dimnary-boolean dimension-nary-helper)
 (displa-def mor	      dimension-nary   " or ")
+(defprop mor dimnary-boolean dimension-nary-helper)
 (displa-def mcond     dim-mcond)
 (displa-def %mcond    dim-mcond)
 
@@ -928,7 +1002,7 @@
 	 (dimension-function form result))
 	(t (setq width (+ 2 width))
 	   (update-heights height depth)
-	   (setq bar `(d-vbar ,height ,depth ,(char (symbol-name $absboxchar) 1)))
+	   (setq bar `(d-vbar ,height ,depth ,(car (coerce $absboxchar 'list))))
 	   (cons bar (nconc arg (cons bar result))))))
 
 (displa-def $matrix dim-$matrix)
@@ -1026,7 +1100,7 @@
      (cond ((not (checkfit (+ 2 width)))
 	    (return (dimension-function (cons '($box) (cdr form)) result))))
      (setq width (+ 2 width) height (1+ height) depth (1+ depth))
-     (setq ch (char (symbol-name $boxchar) 1))
+     (setq ch (car (coerce $boxchar 'list)))
      (setq result
 	   (cons (do ((l (mapcar #'(lambda (l) (char (symbol-name l) 0))
 				 (makstring (caddr form))) (cdr l))
@@ -1046,6 +1120,7 @@
      (return result)))
 
 (displa-def mtext dim-mtext 1 1)
+(defprop mtext dimnary-mtext dimension-nary-helper)
 
 (defun dim-mtext (form result)
   (if (null (cddr form)) (dimension (cadr form) result lop rop 0 0)
@@ -1373,14 +1448,14 @@
 ;; dimension strings.
 
 (defun d-matrix (linear? direction h d)
-  (d-vbar linear? h d (char (symbol-name (if (eq direction 'right)
+  (d-vbar linear? h d (car (coerce (if (eq direction 'right)
 					     $rmxchar
-					     $lmxchar)) 1)))
+					     $lmxchar) 'list))))
 
 ;; There is wired knowledge of character offsets here.
 
 (defun d-box (linear? h d w body &aux (char 0) dmstr) ;char a char?
-  (setq char (char (symbol-name $boxchar) 1))
+  (setq char (car (coerce $boxchar 'list)))
   (setq dmstr `((0 ,h (d-hbar ,(+ 2 w) ,char))
 		(,(- (+ w 2)) 0)
 		(d-vbar ,h ,d ,char)
