@@ -32,12 +32,12 @@
        (setq $flag (setq $dispflag nil))
        (cond ((not (and ($listp $eqns)
 			($listp $vars)))
-	      (simplify ($error (make-mstring "The arguments must both be lists")))))
+	      (simplify ($error "The arguments must both be lists"))))
        (cond ((> ($length $vars)
 		 (setq $l ($length $eqns)))
-	      (simplify ($error (make-mstring "More variables then equations")))))
+	      (simplify ($error "More variables than equations"))))
        (cond ((eql $l 1)
-	      (simplify ($error (make-mstring "Can't eliminate from only one equation")))))
+	      (simplify ($error "Can't eliminate from only one equation"))))
        (cond ((eql ($length $vars) $l)
 	      (setq $vars ($reverse $vars))
 	      (setq $sv (maref $vars 1))
