@@ -25,8 +25,10 @@
     (cond
       ((null x))
       ((cdr x) (mapc #'(lambda (xx) (funcall (get '$grind 'mfexpr*) `(($grind) ,xx))) x))
-      ((symbolp (setq x (strmeval (car x))))
-       (unless (mstringp x) (setq x ($verbify x)))
+      ((or
+         (symbolp (setq x (strmeval (car x))))
+         (and (stringp x) (symbolp (getopr x))))
+       (setq x ($verbify x))
        (cond ((setq y (mget x 'mexpr))
 	      (mgrind (list '(mdefine) (cons (list x) (cdadr y)) (caddr y)) nil))
 	     ((setq y (mget x 'mmacro))
@@ -71,7 +73,7 @@
 		 (mprint (cadr x) out)
 		 (cond ((null (cddr x)) (return nil))
 		       ((and (or (atom (cadr x)) (< (caadr x) (chrct*)))
-			     (or (> (chrct*) (// linel 2))
+			     (or (> (chrct*) (truncate linel 2))
 				 (atom (caddr x)) (< (caaddr x) (chrct*))))
 			(setq i chrps)
 			(mprint (caddr x) out))
@@ -101,7 +103,7 @@
 	      (strprint (cadr x))
 	      (cond ((null (cddr x)) (return nil))
 		    ((and (or (atom (cadr x)) (< (caadr x) (chrct*)))
-			  (or (> (chrct*) (// linel 2))
+			  (or (> (chrct*) (truncate linel 2))
 			      (atom (caddr x)) (< (caaddr x) (chrct*))))
 		     (setq i chrps)
 		     (strprint (caddr x)))
@@ -145,19 +147,20 @@
      (cond ((numberp x) (setq y (exploden x)))
 	   ((and (setq y (safe-get x 'reversealias))
 		 (not (and (member x $aliases :test #'eq) (get x 'noun))))
-	    (setq y (exploden y)))
+	    (setq y (exploden (stripdollar y))))
 	   ((setq y (rassoc x aliaslist :test #'eq)) (return (msize (car y) l r lop rop)))
        ((null (setq y (exploden x))))
        ((safe-get x 'noun) (return (msize-atom (get x 'noun) l r)))
 	   ((char= #\$ (car y)) (setq y (slash (cdr y))))
-	   ((char= #\& (car y))
-	    (do ((l (cdr y) (cdr l))) ((null l))
+	   ((stringp x)
+        (setq y (coerce x 'list))
+	    (do ((l y (cdr l))) ((null l))
 	      (cond ((or (member (car l) '(#\" #\\ #\; #\$) :test #'equal)
 			 (and (char< (car l) #\space)
 			      (not (char= (car l) #\return))))
 		     (rplacd l (cons (car l) (cdr l)))
 		     (rplaca l #\\) (setq l (cdr l)))))
-	    (setq y (cons #\" (nconc (cdr y) (list #\")))))
+	    (setq y (cons #\" (nconc y (list #\")))))
 	   (t (setq y (cons #\? (slash y)))))
      (return (msz y l r))))
 
