@@ -39,9 +39,6 @@
 
 (defun shadow-string-assignment (var value)
   (cond
-    ((mstringp value)
-     (set (get var 'lisp-shadow) (maybe-invert-string-case (symbol-name (stripdollar value))))
-     value)
     ((stringp value)
      (set (get var 'lisp-shadow) value)
      value)
@@ -73,7 +70,8 @@
 	#+gcl "gcl"
 	#+allegro "acl"
 	#+openmcl "openmcl"
-	#-(or clisp cmu scl sbcl gcl allegro openmcl) "unknownlisp")
+    #+abcl "abcl"
+	#-(or clisp cmu scl sbcl gcl allegro openmcl abcl) "unknownlisp")
 
 (defvar $file_search_lisp nil
   "Directories to search for Lisp source code.")
@@ -304,65 +302,76 @@
 	 (demo-patterns "###.{dem,dm1,dm2,dm3,dmt}")
 	 (usage-patterns "##.{usg,texi}")
 	 (share-subdirs-list
-       '("affine"
-	 "algebra"
-	 "algebra/charsets"
-	 "algebra/solver"
-	 "calculus"
-	 "combinatorics"
-	 "contrib"
-	 "contrib/boolsimp"
-	 "contrib/descriptive"
-	 "contrib/diffequations"
-	 "contrib/diffequations/tests"
-	 "contrib/distrib"
-	 "contrib/ezunits"
-	 "contrib/format"
-	 "contrib/fractals"
-	 "contrib/gentran"
-	 "contrib/gentran/test"
-	 "contrib/Grobner"
-     "contrib/levin"
-	 "contrib/lurkmathml"
-	 "contrib/maximaMathML"
-	 "contrib/mcclim"
-	 "contrib/numericalio"
-	 "contrib/pdiff"
-	 "contrib/prim"
-	 "contrib/rand"
-	 "contrib/sarag"
-	 "contrib/simplex"
-	 "contrib/simplex/Tests"
-	 "contrib/solve_rec"
-	 "contrib/state"
-	 "contrib/stats"
-	 "contrib/stringproc"
-	 "contrib/vector3d"
-	 "contrib/unit"
-	 "contrib/Zeilberger"
-	 "diff_form"
-	 "diffequations"
-	 "dynamics"
-	 "draw"
-	 "lapack"
-	 "lbfgs"
-	 "linearalgebra"
-	 "integequations"
-	 "integration"
-	 "macro"
-	 "matrix"
-	 "misc"
-	 "numeric"
-	 "orthopoly"
-	 "physics"
-	 "simplification"
-	 "sym"
-	 "tensor"
-	 "tensor/tests"
-	 "trigonometry"
-	 "utils"
-	 "vector"))
-     ; Smash the list of share subdirs into a string of the form "{affine,algebra,...,vector}" .
+	  ;; It would be nice if we could find these at runtime
+	  ;; instead of hardwiring them here.
+	  '("affine"
+	    "algebra"
+	    "algebra/charsets"
+	    "algebra/solver"
+	    "calculus"
+	    "combinatorics"
+	    "contrib"
+	    "contrib/amatrix"
+	    "contrib/boolsimp"
+	    "contrib/descriptive"
+	    "contrib/diffequations"
+	    "contrib/diffequations/tests"
+	    "contrib/distrib"
+	    "contrib/ezunits"
+	    "contrib/format"
+	    "contrib/fourier_elim"
+	    "contrib/fractals"
+	    "contrib/fresnel"
+	    "contrib/gentran"
+	    "contrib/gentran/test"
+	    "contrib/gf"
+	    "contrib/graphs"
+	    "contrib/Grobner"
+	    "contrib/levin"
+	    "contrib/lurkmathml"
+	    "contrib/maximaMathML"
+	    "contrib/mcclim"
+	    "contrib/namespaces"
+	    "contrib/noninteractive"
+	    "contrib/numericalio"
+	    "contrib/pdiff"
+	    "contrib/prim"
+	    "contrib/rand"
+	    "contrib/sarag"
+	    "contrib/simplex"
+	    "contrib/simplex/Tests"
+	    "contrib/solve_rec"
+	    "contrib/state"
+	    "contrib/stats"
+	    "contrib/stringproc"
+	    "contrib/vector3d"
+	    "contrib/unit"
+	    "contrib/Zeilberger"
+	    "donlp2"
+	    "diff_form"
+	    "diffequations"
+	    "dynamics"
+	    "draw"
+	    "lapack"
+	    "lbfgs"
+	    "linearalgebra"
+	    "integequations"
+	    "integration"
+	    "macro"
+	    "matrix"
+	    "misc"
+	    "numeric"
+	    "orthopoly"
+	    "physics"
+	    "simplification"
+	    "sym"
+	    "tensor"
+	    "tensor/tests"
+	    "trigonometry"
+	    "utils"
+	    "vector"))
+	 ;; Smash the list of share subdirs into a string of the form
+	 ;; "{affine,algebra,...,vector}" .
 	 (share-subdirs (format nil "{~{~A~^,~}}" share-subdirs-list)))
 
     (setq $file_search_lisp
@@ -629,6 +638,19 @@
      (let ((form (read)))
        (prin1 (eval form))))))
 
+(defun maxima-read-eval-print-loop ()
+  (setf *debugger-hook* #'maxima-lisp-debugger-repl)
+  (loop
+   (catch 'to-maxima-repl
+     (format t "~a~%~a> ~a" *prompt-prefix*
+	     (package-name *package*) *prompt-suffix*)
+     (finish-output)
+     (let ((form (read)))
+       (let ((results (multiple-value-list (eval form))))
+	 (dolist (r results)
+	   (fresh-line)
+	   (prin1 r)))))))
+
 (defun maxima-lisp-debugger-repl (condition me-or-my-encapsulation)
   (declare (ignore me-or-my-encapsulation))
   (format t "~&Maxima encountered a Lisp error:~%~% ~A" condition)
@@ -653,7 +675,7 @@
   (do-symbols (s maxima-package)
     (when (and (eql (symbol-package s) maxima-package)
 	       (not (eq s '||))
-	       (member (char (symbol-name s) 0) '(#\$ #\% #\&) :test #'char=))
+	       (member (char (symbol-name s) 0) '(#\$ #\%) :test #'char=))
       (push s *builtin-symbols*)
       (unless (member s problematic-symbols :test #'eq)
 	(setf (gethash s *builtin-symbol-props*)
@@ -668,3 +690,15 @@
 
 (setf *builtin-$props* (copy-list $props))
 (setf *builtin-$rules* (copy-list $rules))
+
+;; Work around: ABCL handles special variables incorrectly
+;; unless they have DEFVAR or DEFPARAMETER.
+
+#+abcl (defvar msg)
+#+abcl (defvar flag)
+#+abcl (defvar print?)
+#+abcl (defvar errcatch)
+#+abcl (defvar mcatch)
+#+abcl (defvar *trigreduce)
+#+abcl (defvar *noexpand)
+#+abcl (defvar $verbose)
