@@ -58,6 +58,9 @@ It appears in LIMIT and DEFINT.......")
 (defmvar limit-answers ()
   "An association list for storing limit answers.")
 
+(defmvar limit-using-taylor ()
+  "Is the current limit computation using taylor expansion?")
+
 (defmvar preserve-direction () "Makes `limit' return Direction info.")
 
 (unless (boundp 'integer-info) (setq integer-info ()))
@@ -130,7 +133,16 @@ It appears in LIMIT and DEFINT.......")
 		     (setq var (second args))
 		       (when ($constantp var)
 			 (merror "Second argument cannot be a constant - `limit'"))
-		       (setq val (third args))
+		       (when (not (or ($subvarp var)
+				      (atom var)))
+			 (merror "Improper limit variable - `limit'"))
+		       (setq val (infsimp (third args)))
+		       ;; infsimp converts -inf to minf.  it also converts -infinity to
+		       ;; infinity, although perhaps this should generate the error below.
+		       (when (and (not (atom val))
+				  (some #'(lambda (x) (not (freeof x val)))
+					infinities))
+			 (merror "Third argument must be a finite value or one of: inf, minf, infinity - `limit'"))
 		       (when (eq val '$zeroa) (setq dr '$plus))
 		       (when (eq val '$zerob) (setq dr '$minus))))
 	      (cond ((= lenargs 4)
@@ -147,6 +159,12 @@ It appears in LIMIT and DEFINT.......")
 		(when (limunknown exp)
 		  (return `((%limit) ,@(cons exp1 (cdr args))))))
 	      (setq varlist (ncons var) genvar nil origval val)
+	      ;; Transform limits to minf to limits to inf by
+	      ;; replacing var with -var everywhere.
+	      (when (eq val '$minf)
+		(setq val '$inf
+		      origval '$inf
+		      exp (subin (m* -1 var) exp)))
 	      ;; Limit is going to want to make its own assumptions
 	      ;; about the variable based on what the calling program
 	      ;; knows. Old assumptions are saved for restoration upon
@@ -161,12 +179,6 @@ It appears in LIMIT and DEFINT.......")
 				((eq dr '$minus) '$zerob)
 				(t 0)))
 		(setq origval 0))
-	      ;; Transform limits to minf to limits to inf by
-	      ;; replacing var with -var everywhere.
-	      (when (eq val '$minf)
-		(setq val '$inf
-		      origval '$inf
-		      exp (subin (m* -1 var) exp)))
 	      (setq exp (resimplify (factosimp (tansc (lfibtophi (limitsimp
 								  ($expand (hide exp) 1 0) var))))))
 	      ;; Resimplify in light of new assumptions.
@@ -188,12 +200,21 @@ It appears in LIMIT and DEFINT.......")
 			 (return (nounlimit exp var val))))
 		    (t (setq exp d)))
 	      (setq ans (limit-catch exp var val))
-	      (if (null ans)
-		  (if (or (real-epsilonp val)
-			  (real-infinityp val))
-		      (return (nounlimit exp var val)))
-		  (return (clean-limit-exp ans)))
-	      (if (setq ans (both-side exp var val))
+
+	      (if (and (null ans)
+		       (not (or (real-epsilonp val)
+				(real-infinityp val))))
+		  (setq ans (both-side exp var val)))
+
+	      ;; try taylor series expansion if simple limit didn't work
+	      (if (and (null ans)		;; if no limit found and
+		       $tlimswitch		;; user says ok to use taylor and
+		       (not limit-using-taylor));; not already doing taylor
+		  (let ((limit-using-taylor t))
+		    (declare (special limit-using-taylor))
+		    (setq ans (limit-catch exp var val))))
+
+	      (if ans
 		  (return (clean-limit-exp ans))
 		  (return (nounlimit exp var val)))))
       (restore-assumptions))))
@@ -209,10 +230,11 @@ It appears in LIMIT and DEFINT.......")
 
 (defun limit-context (var val direction) ;Only works on entry!
   (cond (limit-top
-	 (mapc #'forget (setq global-assumptions (cdr ($facts var))))
+	 (if (atom var)	; declare and facts don't work on subscripted vars
+	     (mapc #'forget (setq global-assumptions (cdr ($facts var)))))
 	 (assume '((mgreaterp) epsilon 0))
-	 (assume '((mlessp) epsilon 1d-8))
-	 (assume '((mgreaterp) prin-inf 1d+8))
+	 (assume '((mlessp) epsilon 1e-8))
+	 (assume '((mgreaterp) prin-inf 1e+8))
 	 (setq limit-assumptions (make-limit-assumptions global-assumptions var val direction))
 	 (setq limit-top ()))
 	(t ()))
@@ -226,9 +248,9 @@ It appears in LIMIT and DEFINT.......")
 	  ((and (not (infinityp val)) (null direction))
 	   ())
 	  ((eq val '$inf)
-	   `(,(assume `((mgreaterp) ,var 1d+8)) ,@new-assumptions))
+	   `(,(assume `((mgreaterp) ,var 1e+8)) ,@new-assumptions))
 	  ((eq val '$minf)
-	   `(,(assume `((mgreaterp) 1d+8 ,var)) ,@new-assumptions))
+	   `(,(assume `((mgreaterp) -1e+8 ,var)) ,@new-assumptions))
 	  ((eq direction '$plus)
 	   `(,(assume `((mgreaterp) ,var 0)) ,@new-assumptions)) ;All limits around 0
 	  ((eq direction '$minus)
@@ -290,8 +312,8 @@ It appears in LIMIT and DEFINT.......")
     (let ((la ($limit exp var val '$plus))
 	  (lb ($limit exp var val '$minus)))
       (cond ((alike1 (ridofab la) (ridofab lb))  (ridofab la))
-	    ((and (not (free la '%limit))
-		  (not (free la '%limit)))  ())
+	    ((or (not (free la '%limit))
+		 (not (free lb '%limit)))  ())
 	    (t '$und)))))
 
 ;; Warning:  (CATCH NIL ...) will catch all throws.
@@ -370,12 +392,15 @@ It appears in LIMIT and DEFINT.......")
 		       (cond ((zerop1 d)
 			      (setq d (behavior a var val))
 			      (if (zerop1 d) (throw 'mabs 'retn))))
-		       (if (or (eq d '$zeroa) (eq d '$inf) (ratgreaterp d 0))
-			   (setq exp (maxima-substitute a `((mabs) ,ans) exp)))
-		       (if (or (eq d '$zerob) (eq d '$minf) (ratgreaterp 0 d))
-			   (setq exp (maxima-substitute (m* -1 a) `((mabs) ,ans) exp)))
 		       (if (eq d '$und)
-			   (throw 'mabs '$und)))
+			   (throw 'mabs d))
+		       (cond ((or (eq d '$zeroa) (eq d '$inf)
+				  (eq d '$ind)
+				  ;; fails on limit(abs(sin(x))/sin(x), x, inf)
+				  (ratgreaterp d 0))
+			      (setq exp (maxima-substitute a `((mabs) ,ans) exp)))
+			     ((or (eq d '$zerob) (eq d '$minf) (ratgreaterp 0 d))
+			      (setq exp (maxima-substitute (m* -1 a) `((mabs) ,ans) exp)))))
 		      (t
 		       (throw 'mabs 'retn))))))))))
 
@@ -405,7 +430,8 @@ It appears in LIMIT and DEFINT.......")
 		(t (simplimit nexp var val))))
 	 ((mexptp nexp)
 	  (cond ((and (eq (cadr nexp) '$inf) (eq (caddr nexp) '$inf)) '$inf)
-		(t (simpinf (m^ '$%e (m* (caddr exp) `((%log) ,(cadr exp))))))))
+		((< infc 2) (simpinf (m^ '$%e (m* (caddr exp) `((%log) ,(cadr exp))))))
+		(t nexp)))
 	 ((< infc 2)  (simpinf nexp))
 	 ((mplusp nexp)
 	  (cond ((member '$infinity (cdr nexp) :test #'eq) '$infinity)
@@ -444,7 +470,7 @@ It appears in LIMIT and DEFINT.......")
 	    (simpinf exp))
 	   (t exp)))
     ((getlimval exp))
-    (t (putlimval exp (cond ((and $tlimswitch
+    (t (putlimval exp (cond ((and limit-using-taylor
 				  (null taylored)
 				  (tlimp exp))
 			     (taylim exp *i*))
@@ -624,7 +650,8 @@ It appears in LIMIT and DEFINT.......")
   (prog (n1 d1 lim-sign gcp sheur-ans)
      (setq n (hyperex n) dn (hyperex dn))
 ;;;Change to uniform limit call.
-     (cond ((infinityp val)  (setq d1 (limit dn var val nil))
+     (cond ((infinityp val)
+	    (setq d1 (limit dn var val nil))
 	    (setq n1 (limit n var val nil)))
 	   (t (cond ((setq n1 (simplimsubst val n)) nil)
 		    (t (setq n1 (limit n var val nil))))
@@ -658,8 +685,7 @@ It appears in LIMIT and DEFINT.......")
 					((equal d1 0) '$und)
 					(t '$ind)))) ;SET LB
 	   ((and (real-infinityp d1) (member n1 '($inf $und $minf) :test #'eq))
-	    (cond ((expfactorp n dn)  (return (expfactor n dn var)))
-		  ((and (not (atom dn)) (not (atom n))
+	    (cond ((and (not (atom dn)) (not (atom n))
 			(cond ((not (equal (setq gcp (gcpower n dn)) 1))
 			       (return (colexpt n dn gcp)))
 			      ((and (eq '$inf val)
@@ -674,23 +700,16 @@ It appears in LIMIT and DEFINT.......")
 	   (t (return (simplimtimes `(,n1 ,(m^ d1 -1))))))
      cp   (setq n ($expand n) dn ($expand dn))
      (cond ((mplusp n)
-	    (let ((maxi-terms (maxi (cdr n))) (new-n ()))
-	      (setq new-n (cond ((not (null (cdr maxi-terms)))
-				 (m+l maxi-terms))
-				(t (car maxi-terms))))
+	    (let ((new-n (m+l (maxi (cdr n)))))
 	      (cond ((not (alike1 new-n n))
-		     (return (limit (m// new-n dn) var '$inf 'think))))
-	      (setq n1 (car maxi-terms))))
+		     (return (limit (m// new-n dn) var val 'think))))
+	      (setq n1 new-n)))
 	   (t (setq n1 n)))
      (cond ((mplusp dn)
-	    (let ((maxi-terms (maxi (cdr dn)))
-		  (new-dn ()))
-	      (setq new-dn (cond ((not (null (cdr maxi-terms)))
-				  (m+l maxi-terms))
-				 (t (car maxi-terms))))
+	    (let ((new-dn (m+l (maxi (cdr dn)))))
 	      (cond ((not (alike1 new-dn dn))
-		     (return (limit (m// n new-dn) var '$inf 'think))))
-	      (setq d1 (car maxi-terms))))
+		     (return (limit (m// n new-dn) var val 'think))))
+	      (setq d1 new-dn)))
 	   (t (setq d1 dn)))
      (setq sheur-ans (sheur0 n1 d1))
      (cond ((or (member sheur-ans '($inf $zeroa) :test #'eq)
@@ -717,50 +736,49 @@ It appears in LIMIT and DEFINT.......")
 	    (return n1)))
      (throw 'limit t)))
 
+;; Test whether both n and dn have form
+;; product of poly^poly
 (defun expfactorp (n dn)
   (do ((llist (append (cond ((mtimesp n) (cdr n))
 			    (t (ncons n)))
 		      (cond ((mtimesp dn) (cdr dn))
 			    (t (ncons dn))))
 	      (cdr llist))
-       (ratexp? t)		  ;IS EVERY ELEMENT SO FAR A POLY^RAT?
-       (one-rat? nil)	  ;IS THERE AT LEAST ONE POLY^RAT WHICH IS NOT
+       (exp? t)		  ;IS EVERY ELEMENT SO FAR
        (factor nil))			;A POLY^POLY?
       ((or (null llist)
-	   (not ratexp?))
-       (and ratexp? one-rat?))
+	   (not exp?))
+       exp?)
     (setq factor (car llist))
-    (setq ratexp? (or (polyp factor)
-		      (and (mexptp factor)
-			   (polyp (cadr factor))
-			   (ratp (caddr factor) var))))
-    (setq one-rat? (or one-rat?
-		       (and (mexptp factor)
-			    (ratp (caddr factor) var)
-			    (not (polyp (caddr factor))))))))
+    (setq exp? (or (polyinx factor var ())
+		   (and (mexptp factor)
+			(polyinx (cadr factor) var ())
+			(polyinx (caddr factor) var ()))))))
 
-(defun expfactor (n dn var)	;ATTEMPS TO EVALUATE LIMIT BY GROUPING
-  (prog (highest-deg)		       ; TERMS WITH SIMILAR EXPONENTS.
-     (let ((new-exp (exppoly n)))	;EXPPOLY UNRATS EXPON
-       (setq n (car new-exp)		;AND RTNS DEG OF EXPONS
+(defun expfactor (n dn var)	;Attempts to evaluate limit by grouping
+  (prog (highest-deg)		       ; terms with similar exponents.
+     (let ((new-exp (exppoly n)))	;exppoly unrats expon
+       (setq n (car new-exp)		;and rtns deg of expons
 	     highest-deg (cdr new-exp)))
-     (cond ((null n) (return nil)))	;NIL MEANS EXPON IS NOT
-     (let ((new-exp (exppoly dn)))	;A RAT FUNC.
+     (cond ((null n) (return nil)))	;nil means expon is not
+     (let ((new-exp (exppoly dn)))	;a rat func.
        (setq dn (car new-exp)
 	     highest-deg (max highest-deg (cdr new-exp))))
-     (cond ((null dn) (return nil)))
+     (cond ((or (null dn)
+		(= highest-deg 0))	; prevent infinite recursion
+	    (return nil)))
      (return
        (do ((answer 1)
 	    (degree highest-deg (1- degree))
 	    (numerator n)
-	    (denomenator dn)
+	    (denominator dn)
 	    (numfactors nil)
 	    (denfactors nil))
 	   ((= degree -1)
 	    (m* answer
-		(limit (m// numerator denomenator)
+		(limit (m// numerator denominator)
 		       var
-		       '$inf
+		       val
 		       'think)))
 	 (let ((newnumer-factor (get-newexp&factors
 				 numerator
@@ -769,21 +787,22 @@ It appears in LIMIT and DEFINT.......")
 	   (setq numerator (car newnumer-factor)
 		 numfactors (cdr newnumer-factor)))
 	 (let ((newdenom-factor (get-newexp&factors
-				 denomenator
+				 denominator
 				 degree
 				 var)))
-	   (setq denomenator (car newdenom-factor)
+	   (setq denominator (car newdenom-factor)
 		 denfactors (cdr newdenom-factor)))
-	 (setq answer (limit (m^ (m* answer
-				     (m// numfactors denfactors))
-				 (cond ((> degree 0) var)
-				       (t 1)))
-			     var
-			     '$inf 'think))
-	 (cond ((eq answer '$und) (return nil))
+	 (setq answer (simplimit (list '(mexpt)
+				       (m* answer
+					   (m// numfactors denfactors))
+				       (cond ((> degree 0) var)
+					     (t 1)))
+				 var
+				 val))
+	 (cond ((member answer '($ind $und) :test #'equal)
+		(return nil))
 	       ((member answer '($inf $minf 0) :test #'equal) ;Really? ZEROA ZEROB?
-		(return answer))
-	       (t nil))))))
+		(return (simplimtimes (list (m// numerator denominator) answer)))))))))
 
 (defun exppoly (exp)	   ;RETURNS EXPRESSION WITH UNRATTED EXPONENTS
   (do ((factor nil)
@@ -856,6 +875,16 @@ It appears in LIMIT and DEFINT.......")
 	   nil)
 	  (t (limit (m// n1 d1) var val 'think)))))
 
+;; takes expression and returns operator at front with all flags removed
+;; except array flag.
+;; array flag must match for alike1 to consider two things to be the same.
+;;   ((MTIMES SIMP) ... ) => (MTIMES)
+;;   ((PSI SIMP ARRAY) 0) => (PSI ARRAY)
+(defun operator-with-array-flag (exp)
+  (cond ((member 'array (car exp) :test #'eq)
+	 (list (caar exp) 'array))
+	(t (list (caar exp)))))
+
 (defun reflect0 (exp var val)
   (cond ((atom exp) exp)
 	((and (eq (caar exp) 'mfactorial)
@@ -864,7 +893,7 @@ It appears in LIMIT and DEFINT.......")
 		    (and (numberp argval)
 			 (> 0 argval)))))
 	 (reflect (cadr exp)))
-	(t (cons (ncons (caar exp))
+	(t (cons (operator-with-array-flag exp)
 		 (mapcar (function
 			  (lambda (term)
 			   (reflect0 term var val)))
@@ -911,6 +940,7 @@ It appears in LIMIT and DEFINT.......")
 		       ((mexpt simp) $z ((mplus simp) ((rat simp) -1 2) $z))
 		       ((mexpt simp) $%e ((mtimes simp) -1 $z)))))
 
+
 (defun no-err-sub (v e &aux ans)
   (let ((errorsw t) (errrjfflag t) (*zexptsimp? t)
 	(errcatch t)
@@ -929,20 +959,25 @@ It appears in LIMIT and DEFINT.......")
     (cond ((null ans) t)     ; Ratfun package returns NIL for failure.
 	  (t ans))))
 
+;; substitute value v for var into expression e.
+;; if result is defined and e is continuous, we have the limit.
 (defun simplimsubst (v e)
-  (prog (ans)
-     (setq ans (no-err-sub (ridofab v) e))
-     (cond ((eq ans t)
-	    (return nil))
-	   ((involve e '(mfactorial)) nil)
-	   ((and (member v '($zeroa $zerob) :test #'eq) (=0 ans))
-	    (setq ans (behavior e var v))
-	    (return (cond ((equal ans 1) '$zeroa)
-			  ((equal ans -1) '$zerob)
-			  (t ans))))
-	   (t (return ans)))))
+  (let (ans)
+    (setq ans (no-err-sub (ridofab v) e))
+    (cond ((eq ans t) nil)
+	  ((involve e '(mfactorial)) nil)
 
-;;;returns (cons numerator denomenator)
+	  ;; functions that are defined at their discontinuities
+	  ((amongl '($atan2 $floor $round $ceiling) e) nil)
+
+	  ((and (member v '($zeroa $zerob) :test #'eq) (=0 ans))
+	   (setq ans (behavior e var v))
+	   (cond ((equal ans 1) '$zeroa)
+		 ((equal ans -1) '$zerob)
+		 (t ans)))
+	  (t ans))))
+
+;;;returns (cons numerator denominator)
 (defun numden* (e)
   (let ((e (factor (simplify e)))
 	(numer ())  (denom ()))
@@ -967,6 +1002,7 @@ It appears in LIMIT and DEFINT.......")
 ;;;Setq's the special vars numer and denom from numden*
 (defun forq (e)
   (cond ((and (mexptp e)
+	      (not (freeof var e))
 	      (null (pos-neg-p (caddr e))))
 	 (setq denom (cons (m^ (cadr e) (m* -1. (caddr e))) denom)))
 	(t (setq numer (cons e numer)))))
@@ -1028,7 +1064,7 @@ It appears in LIMIT and DEFINT.......")
 			    (limroot x power))
 			(cdr exp))))))
 
-;;NUMERATOR AND DENOMENATOR HAVE EXPONENTS WITH GCD OF GCP.
+;;NUMERATOR AND DENOMINATOR HAVE EXPONENTS WITH GCD OF GCP.
 ;;; Used to call simplimit but some of the transformations used here
 ;;; were not stable w.r.t. the simplifier, so try keeping exponent separate
 ;;; from bas.
@@ -1262,11 +1298,18 @@ It appears in LIMIT and DEFINT.......")
     (setq lhp? (and (null ind) (cons n d)))
     (desetq (nconst . n) (var-or-const n))
     (desetq (dconst . d) (var-or-const d))
-    (setq n (sdiff n var) d (sdiff d var))
+
+    (setq n (stirling0 n))	;; replace factorial and %gamma
+    (setq d (stirling0 d))  	;;  with approximations
+    
+    (setq n (sdiff n var)	;; take derivatives for l'hospital
+	  d (sdiff d var))
+
     (if (or (not (free n '%derivative)) (not (free d '%derivative)))
 	(throw 'lhospital ()))
     (setq n (expand-trigs (tansc n) var))
     (setq d (expand-trigs (tansc d) var))
+
     (desetq (const . (n . d)) (remove-singularities n d))
     (setq const (m* const (m// nconst dconst)))
     (simpinf
@@ -1550,12 +1593,18 @@ It appears in LIMIT and DEFINT.......")
     ((eq bl '$ind)  (cond ((or (zerop2 el) (infinityp el)) '$und)
 			  ((not (equal (getsignl el) -1)) '$ind)
 			  (t '$und)))
-    ((eq el '$inf)  (cond ((abless1 bl)
+    ((eq el '$inf)  (cond ((abeq1 bl)
+			   (cond ((equal (getsignl bl) 1) 1)
+				 (t '$ind)))
+			  ((abless1 bl)
 			   (cond ((equal (getsignl bl) 1) '$zeroa)
 				 (t 0)))
 			  ((equal (getsignl bl) -1) '$infinity)
 			  (t '$inf)))
-    ((eq el '$minf)  (cond ((not (abless1 bl))
+    ((eq el '$minf)  (cond ((abeq1 bl)
+			    (cond ((equal (getsignl bl) 1) 1)
+				  (t '$ind)))
+			   ((not (abless1 bl))
 			    (cond ((equal (getsignl bl) 1)  '$zeroa)
 				  (t 0)))
 			   ((ratgreaterp 0 bl)  '$infinity)
@@ -1579,12 +1628,20 @@ It appears in LIMIT and DEFINT.......")
   (cond ((numberp x) (and (integerp x) (evenp x)))
 	((and (mnump x) (evenp (cadr x))))))
 
+;; is absolute value less than one?
 (defun abless1 (bl)
   (setq bl (nmr bl))
   (cond ((mnump bl)
 	 (and (ratgreaterp 1. bl) (ratgreaterp bl -1.)))
 	(t (equal (getsignl (m1- `((mabs) ,bl))) -1.))))
 
+;; is absolute value equal to one?
+(defun abeq1 (bl)
+  (setq bl (nmr bl))
+  (cond ((mnump bl)
+	 (or (equal 1. bl) (equal bl -1.)))
+	(t (equal (getsignl (m1- `((mabs) ,bl))) 0))))
+
 (defmfun simplimit (exp var val)
   (cond
     ((eq var exp) val)
@@ -1644,14 +1701,11 @@ It appears in LIMIT and DEFINT.......")
 		   (cdr exp)))
      exp)				;LIMIT(B[I],B,INF); -> B[I]
     (t (if $limsubst
-	   (let ((head (cond ((member 'array (car exp) :test #'eq)
-			      (list (caar exp) 'array))
-			     (t (list (caar exp))))))
-	     (simplify (cons head
-			     (mapcar #'(lambda (a)
-					 (limit a var val 'think))
-				     (cdr exp)))))
-	   (nounlimit exp var val)))))
+	   (simplify (cons (operator-with-array-flag exp)
+			   (mapcar #'(lambda (a)
+				       (limit a var val 'think))
+				   (cdr exp))))
+	   (throw 'limit t)))))
 
 (defun liminv (e)
   (setq e (resimplify (subst (m// 1 var) var e)))
@@ -1662,6 +1716,13 @@ It appears in LIMIT and DEFINT.......")
 
 (defun simplimtimes (exp)
   (prog (sign prod y num denom flag zf flag2 exp1)
+     (if (expfactorp (cons '(mtimes) exp) 1)
+	 ;; handles        (-1)^x * 2^x => (-2)^x => $infinity
+	 ;; want to avoid  (-1)^x * 2^x => $ind * $inf => $und
+	 (let ((ans (expfactor (cons '(mtimes) exp) 1 var)))
+	   (if ans
+	       (return ans))))
+	
      (setq prod (setq num (setq denom 1)) exp1 exp)
      loop
      (setq y (let ((loginprod? (involve (car exp1) '(%log))))
@@ -1750,7 +1811,9 @@ It appears in LIMIT and DEFINT.......")
      (do ((exp (cdr exp) (cdr exp)) (f))
 	 ((or y (null exp)) nil)
        (setq f (limit (car exp) var val 'think))
-       (cond ((eq f '$und) (setq y t))
+       (cond ((null f)
+	      (throw 'limit t))
+	     ((eq f '$und) (setq y t))
 	     ((not (member f '($inf $minf $infinity $ind) :test #'eq))
 	      (setq sum (m+ sum f)))
 	     ((eq f '$ind)  (push (car exp) indl))
@@ -1925,8 +1988,8 @@ It appears in LIMIT and DEFINT.......")
 
 (defun sheur1 (l1 l2)
   (prog (ans)
-     (setq l1 (car (maxi l1)))
-     (setq l2 (car (maxi l2)))
+     (setq l1 (m+l (maxi l1)))
+     (setq l2 (m+l (maxi l2)))
      (setq ans (cpa l1 l2 t))
      (return (cond ((=0 ans)  (m+  l1 l2))
 		   ((equal ans 1.) '$inf)
@@ -1994,7 +2057,7 @@ It appears in LIMIT and DEFINT.......")
 		(or (equal ($radcan (m// (cadr ans) (cadr d))) 1.)
 		    (and (polyp (cadr ans))
 			 (polyp (cadr d))
-			 (equal (limit (m// (cadr ans) (cadr d)) var '$inf 'think)
+			 (equal (limit (m// (cadr ans) (cadr d)) var val 'think)
 				1.))))
 	   (let ((new-term1 (m// t1 (cadr ans)))
 		 (new-term2 (m// t2 (cadr d))))
@@ -2083,7 +2146,9 @@ It appears in LIMIT and DEFINT.......")
 	  ((and (eq ta tb) (eq ta 'exp))
 	   ;; Both are exponential order of infinity.  Check the
 	   ;; exponents to determine which exponent is bigger.
-	   (ratgreaterp (third (second a)) (third (second b))))
+	   (ratgreaterp (limit (m- `((%log) ,(second a)) `((%log) ,(second b)))
+			       var val 'think)
+			0))
 	  ((member ta (cdr (member tb '(num log var exp fact gen) :test #'eq)) :test #'eq)))))
 
 (defun ismax (l)
@@ -2127,25 +2192,33 @@ It appears in LIMIT and DEFINT.......")
 		 (t (setq ans ())))))
 	(t ())))
 
-(defun maxi (l)				;RETURNS LIST OF HIGH TERMS
-  (cond ((atom l)  nil)
-	(t (do ((l (cdr l) (cdr l))
-		(hi-term (car l))
-		(hi-terms (ncons (car l)))
+;RETURNS LIST OF HIGH TERMS
+(defun maxi (all)
+  (cond ((atom all)  nil)
+	(t (do ((l (cdr all) (cdr l))
+		(hi-term (car all))
+		(total 1)		; running total constant factor of hi-term
+		(hi-terms (ncons (car all)))
 		(compare nil))
-	       ((null l) hi-terms)
+	       ((null l) (if (zerop2 total)  ; if high-order terms cancel each other
+			     all	     ; keep everything
+			   hi-terms))        ; otherwise return list of high terms
 	     (setq compare (limit (m// (car l) hi-term) var val 'think))
 	     (cond
 	       ((infinityp compare)
+		(setq total 1)
 		(setq hi-terms (ncons (setq hi-term (car l)))))
 	       ((eq compare '$und)
 		(let ((compare2 (limit (m// hi-term (car l)) var val 'think)))
 		  (cond ((zerop2 compare2)
+			 (setq total 1)
 			 (setq hi-terms (ncons (setq hi-term (car l)))))
 			(t nil))))
 	       ((zerop2 compare)  nil)
-;;;COMPARE IS IND OR FINITE-VALUED
-	       (t (setq hi-terms (append hi-terms (ncons (car l))))))))))
+	       ;; COMPARE IS IND OR FINITE-VALUED
+	       (t
+		(setq total (m+ total compare))
+		(setq hi-terms (append hi-terms (ncons (car l))))))))))
 
 (defun ratmax (l)
   (prog (ans)
@@ -2192,6 +2265,7 @@ It appears in LIMIT and DEFINT.......")
      (setq l (cdr l))
      (go loop)))
 
+; returns rough class-of-growth of term
 (defun istrength (term)
   (cond ((mnump term)  (list 'num term))
 	((atom term)   (cond ((eq term var)
@@ -2213,21 +2287,22 @@ It appears in LIMIT and DEFINT.......")
 		 (t `(gen ,temp)))))
 	((and (mexptp term)
 	      (real-infinityp (limit term var val t)))
-	 (cond ((and (among var (caddr term))
-		     (member (car (istrength (setq term (logred term))))
-			   '(var exp fact) :test #'eq)
-		     (real-infinityp (limit term var val t)))
-		(list 'exp (m^ '$%e term)))
-	       ((not (among var (caddr term)))
-		(let ((temp (istrength (cadr term))))
-		  (cond ((not (alike1 temp term))
-			 (rplaca (cdr temp) term)
-			 (and (eq (car temp) 'var)
-			      (rplaca (cddr temp)
-				      (m* (caddr temp) (caddr term))))
-			 temp)
-			(t `(gen ,term)))))
-	       (t (list 'gen (m^ '$%e term)))))
+	 (let ((logterm (logred term)))
+	   (cond ((and (among var (caddr term))
+		       (member (car (istrength logterm))
+			       '(var exp fact) :test #'eq)
+		       (real-infinityp (limit logterm var val t)))
+		  (list 'exp (m^ '$%e logterm)))
+		 ((not (among var (caddr term)))
+		  (let ((temp (istrength (cadr term))))
+		    (cond ((not (alike1 temp term))
+			   (rplaca (cdr temp) term)
+			   (and (eq (car temp) 'var)
+				(rplaca (cddr temp)
+					(m* (caddr temp) (caddr term))))
+			   temp)
+			  (t `(gen ,term)))))
+		 (t `(gen ,term)))))
 	((and (eq (caar term) '%log)
 	      (real-infinityp (limit term var val t)))
 	 (let ((stren (istrength (cadr term))))
@@ -2243,6 +2318,7 @@ It appears in LIMIT and DEFINT.......")
 		(istrength temp))))
 	(t (list 'gen term))))
 
+;; log reduce - returns log of s1
 (defun logred (s1)
   (or (and (eq (cadr s1) '$%e) (caddr s1))
       (m*t (caddr s1) `((%log) ,(cadr s1)))))
@@ -2321,7 +2397,7 @@ It appears in LIMIT and DEFINT.......")
 		 (mapcar #'rdsget (cdr e))))))
 
 (defun rdtay (rd)
-  (cond ($tlimswitch ($ratdisrep ($taylor rd var val 1.)))
+  (cond (limit-using-taylor ($ratdisrep ($taylor rd var val 1.)))
 	(t (lrdtay rd))))
 
 (defun lrdtay (rd)
@@ -2386,7 +2462,8 @@ It appears in LIMIT and DEFINT.......")
   (let ((num-power (rddeg n nil))
 	(den-power (rddeg d nil))
 	(coef ())  (coef-sign ())  (power ()))
-    (setq coef (m// ($ratcoef n var num-power) ($ratcoef d var den-power)))
+    (setq coef (m// ($ratcoef ($expand n) var num-power)
+		    ($ratcoef ($expand d) var den-power)))
     (setq coef-sign (getsignl coef))
     (setq power (m// num-power den-power))
     (cond ((eq (ask-integer power '$integer) '$integer)
@@ -2404,7 +2481,7 @@ It appears in LIMIT and DEFINT.......")
 	  ((ratgreaterp num-power den-power)
 	   (cond ((equal coef-sign 1.)  '$inf)
 		 ((equal coef-sign -1)  '$minf)
-		 ((equal coef-sign 0)   0) ;Questionable!
+		 ((equal coef-sign 0)   nil) ; should never be zero
 		 ((null coef-sign)   '$infinity)))
 	  (t coef))))
 
@@ -2458,12 +2535,15 @@ It appears in LIMIT and DEFINT.......")
   ;; it's not identically zero, we compute the limit of the real and
   ;; imaginary parts and combine them.  Otherwise, we can use the
   ;; original method for real limits.
-  (let* ((log-form `((%log) ,arg))
-	 (rp ($realpart log-form))
-	 (ip (simplify ($imagpart log-form))))
+  (destructuring-let* ((arglim (limit arg var val 'think))
+		       (log-form `((%log) ,arg))
+		       ((rp . ip) (if (or (and (mnump arglim)
+					       (ratgreaterp arglim 0))
+					  (eq arglim '$inf))	; if limit is real pos
+				      (cons arglim 0)		; avoid asking user q's
+				    (trisplit log-form))))	; otherwise find real part
     (cond ((and (numberp ip) (zerop ip))
-	   (let* ((arglim (limit arg var val 'think))
-		  (real-lim (ridofab arglim)))
+	   (let* ((real-lim (ridofab arglim)))
 	     (if (=0 real-lim)
 		 (cond ((eq arglim '$zeroa)  '$minf)
 		       ((eq arglim '$zerob)  '$infinity)
@@ -2539,6 +2619,7 @@ It appears in LIMIT and DEFINT.......")
   (cond ((real-infinityp arg)
 	 (cond ((eq sch '%sinh) arg) (t '$inf)))
 	((eq arg '$infinity) '$infinity)
+	((eq arg '$ind) '$ind)
 	((eq arg '$und) '$und)
 	(t (let (($exponentialize t))
 	     (resimplify (list (ncons sch) (ridofab arg)))))))
