@@ -17,7 +17,7 @@
   (cond ((atom exp) exp)
 	((arcp (caar exp)) (logarc (caar exp) ($logarc (cadr exp))))
 	((eq (caar exp) '$atan2)
-	 (logarc '%atan ($logarc (div (cadr exp) (caddr exp)))))
+	 (logarc '%atan2 (list ($logarc (second exp)) ($logarc (third exp)))))
 	(t (recur-apply #'$logarc exp))))
 
 (defmfun logarc (f x)
@@ -33,6 +33,14 @@
 	 ;; (log(1 + %i*x) - log(1 - %i*x)) /(2 %i)
 	 (div (sub (take '(%log) (add 1 (mul '$%i x))) (take '(%log) (sub 1 (mul '$%i x))))
 	      (mul 2 '$%i)))
+	((eq f '%atan2)
+	 ;; atan2(y,x) = -%i*log((x + %i*y)/sqrt(x^2+y^2))
+	 (destructuring-bind (y x)
+	     x
+	   (mul -1 '$%i
+		(take '(%log) (div (add x (mul '$%i y))
+				   (power (add (mul x x) (mul y y))
+					  1//2))))))
     	((eq f '%asinh)
 	 ;; log(sqrt(x^2+1)+x)
 	 (take '(%log) (add x (root (add 1 (power x 2)) 2))))
@@ -42,7 +50,7 @@
     	((eq f '%atanh)
 	 ;;  (log(x+1)-log(1-x))/2
 	 (div (sub (take '(%log) (add 1 x)) (take '(%log) (sub 1 x))) 2))
-    	((memq f '(%asec %acsc %acot %asech %acsch %acoth))
+    	((member f '(%asec %acsc %acot %asech %acsch %acoth) :test #'eq)
 	 ;; asec(x) = acos(1/x), and etc.
 	 (logarc (oldget (oldget (get f '$inverse) 'recip) '$inverse) (inv x)))
 	(t (merror "Bad argument to 'logarc'"))))
@@ -54,15 +62,15 @@
        (halfangleaux f (mul 2 a))))
 
 (defun halfangleaux (f a) ;; f=function; a=twice argument
-  (let ((sw (memq f '(%cos %cot %coth %cosh))))
-    (cond ((memq f '(%sin %cos))
+  (let ((sw (member f '(%cos %cot %coth %cosh) :test #'eq)))
+    (cond ((member f '(%sin %cos) :test #'eq)
 	   (power (div (add 1 (porm sw (take '(%cos) a))) 2) (1//2)))
-	  ((memq f '(%tan %cot))
+	  ((member f '(%tan %cot) :test #'eq)
 	   (div (add 1 (porm sw (take '(%cos) a))) (take '(%sin) a)))
-	  ((memq f '(%sinh %cosh))
+	  ((member f '(%sinh %cosh) :test #'eq)
 	   (power (div (add (take '(%cosh) a) (porm sw 1)) 2) (1//2)))
-	  ((memq f '(%tanh %coth))
+	  ((member f '(%tanh %coth) :test #'eq)
 	   (div (add (take '(%cosh) a) (porm sw 1)) (take '(%sinh) a)))
-	  ((memq f '(%sec %csc %sech %csch))
+	  ((member f '(%sec %csc %sech %csch) :test #'eq)
 	   (inv (halfangleaux (get f 'recip) a))))))
 
