@@ -185,7 +185,7 @@
 	(unless $nolabels (set d-tag $%))
 	(setq $_ $__)
 	(when $showtime	;; we don't distinguish showtime:all?? /RJF
-	  (format t "Evaluation took ~$ seconds (~$ elapsed)"
+	  (format t "Evaluation took ~,4F seconds (~,4F elapsed)"
 		  time-used etime-used )
 	  #+gcl
 	  (format t "~%")
@@ -295,7 +295,7 @@
 
 (defun batch (filename &optional demo-p
 	      &aux (orig filename) list
-	      file-obj (accumulated-time 0d0) (abortp t))
+	      file-obj (accumulated-time 0.0) (abortp t))
   (setq list (if demo-p '$file_search_demo '$file_search_maxima))
   (setq filename ($file_search filename (symbol-value list)))
   (or filename (merror "Could not find ~M in ~M: ~M"
@@ -305,7 +305,7 @@
        (progn (batch-internal (setq file-obj (open filename)) demo-p)
 	      (setq abortp nil)
 	      (when $showtime
-		(format t "~&Batch spent ~$ seconds in evaluation.~%"
+		(format t "~&Batch spent ~,4F seconds in evaluation.~%"
 			accumulated-time)))
     (if file-obj (close file-obj))
     (when abortp (format t "~&(Batch of ~A aborted.)~%" filename))))
@@ -394,8 +394,7 @@
   #+gcl (format t " (aka GCL)")
   (format t "~%Distributed under the GNU Public License. See the file COPYING.~%")
   (format t "Dedicated to the memory of William Schelter.~%")
-  (format t "This is a development version of Maxima. The function bug_report()~%")
-  (format t "provides bug reporting information.~%"))
+  (format t "The function bug_report() provides bug reporting information.~%"))
 
 #+kcl
 (si::putprop :t 'throw-macsyma-top 'si::break-command)
@@ -411,7 +410,7 @@
 
 (defmfun $appendfile (name)
   (if (and (symbolp name)
-	   (member (char (symbol-name name) 0) '(#\& #\$) :test #'char=))
+	   (member (char (symbol-name name) 0) '(#\$) :test #'char=))
       (setq name (maxima-string name)))
   (if $appendfile (merror "already in appendfile, use closefile first"))
   (let ((stream  (open name :direction :output
@@ -480,8 +479,7 @@
   (let ((ans "") )
     (dolist (v x)
       (setq ans (concatenate 'string ans
-			     (cond ((and (symbolp v) (char= (char (symbol-name v) 0) #\&))
-				    (subseq (print-invert-case v) 1))
+			     (cond
 				   ((stringp v) v)
 				   (t
 				    (coerce (mstring v) 'string))))))
@@ -497,7 +495,7 @@
 			      (list "-c" (apply '$sconcat args)) :output t)
   #+openmcl (ccl::run-program "/bin/sh"
 			      (list "-c" (apply '$sconcat args)) :output t)
-  )
+  #+abcl (extensions::run-shell-command (apply '$sconcat args)))
 
 (defun $room (&optional (arg nil arg-p))
   (if arg-p
@@ -511,7 +509,7 @@
   (throw 'return-from-debugger t))
 
 (let ((t0-real 0) (t0-run 0)
-      (float-units (float internal-time-units-per-second 1d0)))
+      (float-units (float internal-time-units-per-second)))
 
   (defun initialize-real-and-run-time ()
     (setq t0-real (get-internal-real-time))
