@@ -5,8 +5,10 @@
 ($auto_mexpr '$unsum "nusum")
 ($auto_mexpr '$funcsolve "nusum")
 
+($auto_mexpr '$bffac "bffac")
 ($auto_mexpr '$bfzeta "bffac")
 ($auto_mexpr '$bfpsi "bffac")
+($auto_mexpr '$bfpsi0 "bffac")
 
 (auto-mexpr '$trigrat '|trigrat|)
 ($auto_mexpr '$gcdex '|gcdex|)
@@ -25,7 +27,7 @@
 ($auto_mexpr '$declare_linear_operator "misc/declin")
 
 ($auto_mexpr '$nonumfactor "simplification/genut")
-(meval '((%setup_autoload simp) &bffac $bfzeta))
+(meval '((%setup_autoload simp) "bffac" $bfzeta))
 
 ;jfa
 ($auto_mexpr '$eigenvectors '|eigen|)
@@ -37,6 +39,7 @@
 ($auto_mexpr '$ic2 "ode2.mac")
 ($auto_mexpr '$bc2 "ode2.mac")
 ($auto_mexpr '$desimp "ode2.mac")
+($auto_mexpr '$linear2 "ode2")
 
 (dolist (v       
 	  '($arite
@@ -124,12 +127,9 @@
        $lcharp
        $lowercasep
        $uppercasep
-       $sunlisp
-       $lstringp
        $stringp
        $charat
        $charlist
-       $parsetoken
        $sconc
        $scopy
        $sdowncase
@@ -156,20 +156,30 @@
        $strimr
        $substring
        $supcase
+       $tab
        $tokens ))
   (setf (get f 'autoload) "stringproc"))
 
 
-(auto-mspec '$romberg "romberg")
+(setf (get '$romberg 'autoload) "romberg")
 
-(dolist (f       
-     '($read_matrix
-       $read_lisp_array
-       $read_maxima_array
-       $read_hashed_array
-       $read_nested_list
-       $read_list
-       $write_data ))
+(dolist (f
+  '($assume_external_byte_order
+    $opena_binary
+    $openr_binary
+    $openw_binary
+    $read_array
+    $read_binary_array
+    $read_binary_list
+    $read_binary_matrix
+    $read_hashed_array
+    $read_lisp_array
+    $read_list
+    $read_matrix
+    $read_maxima_array
+    $read_nested_list
+    $write_binary_data
+    $write_data))
   (setf (get f 'autoload) "numericalio"))
 
 (setf (get '$eval_string 'autoload) "eval_string")
@@ -187,6 +197,7 @@
        $hessian
        $jacobian
        $matrix_sign
+       $vandermonde_matrix
        
        $blockmatrixp           ; linalg-utilities.lisp
        $ctranspose
@@ -246,14 +257,13 @@
        $polytocompanion
        $ptriangularize
        $ptriangularize_with_proviso
-       $rank
+       $linalg_rank
        $request_rational_matrix
        $require_integer
        $require_symbol
        $rowop
        $rowswap
-       $toeplitz
-       $vandermonde_matrix ))
+       $toeplitz))
   ($auto_mexpr mexpr "linearalgebra"))
 
 
