@@ -47,11 +47,13 @@
   (cond ((not (and ($ratp taylor-form) (member 'trunc (car taylor-form) :test #'eq)))
 	 (merror "First argument to Pade must be a taylor series")))
   (destructuring-let (((nil nil varlist genvar tlist) (car taylor-form)))
-    (cons '(mlist)
-	  (pade (cdr taylor-form)
-		(cons (car (tay-order n-bound))
-		      (car (tay-order d-bound)))
-		(orig-trunc (car (mrat-tlist taylor-form)))))))
+    (if (psp (cdr taylor-form))		     
+	(cons '(mlist)
+	      (pade (cdr taylor-form)
+		    (cons (car (tay-order n-bound))
+			  (car (tay-order d-bound)))
+		    (orig-trunc (car (mrat-tlist taylor-form)))))
+      (list '(mlist) (srdis (cdr taylor-form))))))
 	     
 (defun pade (tpf bounds trunc)
   (let ((ord (le (terms tpf))) (poly (terms tpf)))
